@@ -1,19 +1,21 @@
 /* emacs objective-c mode -*- objc -*- */
 
 #import "AgendaStore.h"
-#import "AppointmentEditor.h"
-#import "TaskEditor.h"
 #import "CalendarView.h"
 #import "DayView.h"
+#import "WeekView.h"
 #import "Element.h"
 #import "Event.h"
 #import "PreferencesController.h"
 #import "DataTree.h"
+#import "SelectionManager.h"
+#import "ConfigManager.h"
 
-@interface AppController : NSObject <AgendaDataSource>
+@interface AppController : NSObject <ConfigListener>
 {
   IBOutlet CalendarView *calendar;
   IBOutlet DayView *dayView;
+  IBOutlet WeekView *weekView;
   IBOutlet NSOutlineView *summary;
   IBOutlet NSTextField *search;
   IBOutlet NSTableView *taskView;
@@ -21,12 +23,8 @@
   IBOutlet NSTabView *tabs;
 
   PreferencesController *_pc;
-  AppointmentEditor *_editor;
-  TaskEditor *_taskEditor;
   StoreManager *_sm;
-  Event *_selection;
-  Element *_clickedElement;
-  BOOL _deleteSelection;
+  SelectionManager *selectionManager;
   Date *_selectedDay;
   DataTree *_summaryRoot;
   DataTree *_today;
@@ -34,6 +32,7 @@
   DataTree *_soon;
   DataTree *_results;
   DataTree *_tasks;
+  NSView *_appicon;
 }
 
 - (void)copy:(id)sender;
@@ -52,5 +51,11 @@
 - (void)exportAppointment:(id)sender;
 - (void)doSearch:(id)sender;
 - (void)clearSearch:(id)sender;
+- (void)today:(id)sender;
+- (void)nextDay:(id)sender;
+- (void)previousDay:(id)sender;
+- (void)nextWeek:(id)sender;
+- (void)previousWeek:(id)sender;
 - (void)updateSummaryData;
+- (void)dataChanged:(NSNotification *)not;
 @end
