@@ -2,17 +2,64 @@
 
 #import <AppKit/AppKit.h>
 #import "AppointmentEditor.h"
+#import "TaskEditor.h"
 #import "StoreManager.h"
 #import "AppController.h"
 #import "Event.h"
 #import "Task.h"
 #import "PreferencesController.h"
 #import "iCalTree.h"
+#import "SelectionManager.h"
+#import "AlarmManager.h"
+#import "defines.h"
 
-NSComparisonResult compareAppointments(id a, id b, void *data)
+@interface AppIcon : NSView
 {
-  return [[a startDate] compareTime:[b startDate]];
+  NSTimer *timer;
 }
+@end
+@implementation AppIcon
+- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent
+{
+  return YES;
+}
+- (void)secondChanged:(NSTimer *)timer
+{
+  [self setNeedsDisplay:YES];
+}
+- (void)drawRect:(NSRect)rect
+{
+  ConfigManager *config = [ConfigManager globalConfig];
+  NSCalendarDate *now = [[Date now] calendarDate];
+  NSDictionary *attrs;
+  NSString *aString;
+
+  attrs = [NSMutableDictionary dictionaryWithObject:[NSFont systemFontOfSize: 8]  forKey:NSFontAttributeName];
+  if ([[config objectForKey:APPICON_DATE] boolValue]) {
+    aString = [now descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
+    [aString drawAtPoint:NSMakePoint(8, 3) withAttributes:attrs];
+  }
+  if ([[config objectForKey:APPICON_TIME] boolValue]) {
+    aString = [now descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSTimeFormatString]];
+    [aString drawAtPoint:NSMakePoint(11, 48) withAttributes:attrs];
+    if (timer == nil)
+      timer = [NSTimer scheduledTimerWithTimeInterval:1
+         			       	       target:self
+		                             selector:@selector(secondChanged:)
+				             userInfo:nil
+				              repeats:YES];
+  } else {
+    if (timer)
+      [timer invalidate];
+    timer = nil;
+  }
+}
+- (void)mouseDown:(NSEvent *)theEvent
+{
+  if ([theEvent clickCount] > 1)
+    [NSApp unhide:self];
+}
+@end
 
 NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
@@ -22,18 +69,17 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 @implementation AppController
 - (void)registerForServices
 {
-  NSArray *sendTypes = [NSArray arrayWithObjects:NSStringPboardType, NSFilenamesPboardType, nil];
-  NSArray *returnTypes = [NSArray arrayWithObjects:nil];
-  [NSApp registerServicesMenuSendTypes: sendTypes returnTypes: returnTypes];
+  [NSApp registerServicesMenuSendTypes: [NSArray arrayWithObjects:NSStringPboardType, NSFilenamesPboardType, nil]
+	                   returnTypes: [NSArray arrayWithObjects:nil]];
 }
 
 - (void)initSummary
 {
-  _today = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:@"Today" forKey:@"title"]];
-  _tomorrow = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:@"Tomorrow" forKey:@"title"]];
-  _soon = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:@"Soon" forKey:@"title"]];
-  _results = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:@"Search results" forKey:@"title"]];
-  _tasks = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:@"Open tasks" forKey:@"title"]];
+  _today = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:_(@"Today") forKey:@"title"]];
+  _tomorrow = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:_(@"Tomorrow") forKey:@"title"]];
+  _soon = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:_(@"Soon") forKey:@"title"]];
+  _results = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:_(@"Search results") forKey:@"title"]];
+  _tasks = [DataTree dataTreeWithAttributes:[NSDictionary dictionaryWithObject:_(@"Open tasks") forKey:@"title"]];
   _summaryRoot = [DataTree new];
   [_summaryRoot addChild:_today];
   [_summaryRoot addChild:_tomorrow];
@@ -45,17 +91,19 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (NSDictionary *)attributesFrom:(Event *)event and:(Date *)date
 {
   Date *today = [Date today];
+  Date *copy = AUTORELEASE([date copy]);
   NSMutableDictionary *attributes = [NSMutableDictionary new];
   NSString *details;
   NSString *title;
 
-  [date setMinute:[[event startDate] minuteOfDay]];
+  [copy setIsDate:NO];
+  [copy setMinute:[[event startDate] minuteOfDay]];
   [attributes setValue:event forKey:@"object"];
-  [attributes setValue:AUTORELEASE([date copy]) forKey:@"date"];
-  if ([today daysUntil:date] > 0 || [today daysSince:date] > 0)
-    details = [[date calendarDate] descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
+  [attributes setValue:copy forKey:@"date"];
+  if ([today timeIntervalSinceDate:copy] > 86400 || [today timeIntervalSinceDate:copy] < -86400)
+    details = [[copy calendarDate] descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
   else
-    details = [[date calendarDate] descriptionWithCalendarFormat:@"%H:%M"];
+    details = [[copy calendarDate] descriptionWithCalendarFormat:@"%H:%M"];
   title = [NSString stringWithFormat:@"%@ : %@", details, [event summary]];
   [attributes setValue:title forKey:@"title"];
   return AUTORELEASE(attributes);
@@ -104,21 +152,41 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
     if ([task state] != TK_COMPLETED)
       [_tasks addChild:[DataTree dataTreeWithAttributes:[self attributesFromTask:task]]];
   }
+  [_tasks setValue:[NSString stringWithFormat:_(@"Open tasks (%d)"), [[_tasks children] count]] forKey:@"title"];
   [summary reloadData];
 }
 
+- (void)setWindowTitle
+{
+  NSTabViewItem *dayTab = [tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Day"]];
+  NSTabViewItem *weekTab = [tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Week"]];
+
+  if ([tabs selectedTabViewItem] == dayTab)
+    [window setTitle:[NSString stringWithFormat:@"SimpleAgenda - %@", [calendar dateAsString]]];
+  else if ([tabs selectedTabViewItem] == weekTab)
+    [window setTitle:[@"SimpleAgenda - " stringByAppendingString:[NSString stringWithFormat:_(@"Week %d"), [_selectedDay weekOfYear]]]];
+  else
+    [window setTitle:[@"SimpleAgenda - " stringByAppendingString:_(@"Tasks")]];
+}
+
+- (NSDictionary *)defaults
+{
+  return [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES], [NSNumber numberWithBool:YES], [NSNumber numberWithBool:NO], nil]
+		       forKeys:[NSArray arrayWithObjects:APPICON_DATE, APPICON_TIME, TOOLTIP, nil]];
+}
+
 - (id)init
 {
   self = [super init];
   if (self) {
-    ASSIGNCOPY(_selectedDay, [Date today]);
-    _selection = nil;
-    _editor = [AppointmentEditor new];
-    _taskEditor = [TaskEditor new];
-    _sm = [StoreManager new];
-    _pc = [[PreferencesController alloc] initWithStoreManager:_sm];
+    ConfigManager *config = [ConfigManager globalConfig];
+    [config registerDefaults:[self defaults]];
+    [config registerClient:self forKey:APPICON_DATE];
+    [config registerClient:self forKey:APPICON_TIME];
+    selectionManager = [SelectionManager globalManager];
+    _sm = [StoreManager globalManager];
+    _pc = [PreferencesController new];
     [self initSummary];
-    [self registerForServices];
   }
   return self;
 }
@@ -128,49 +196,79 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   NSPopUpButtonCell *cell = [NSPopUpButtonCell new];
   [cell addItemsWithTitles:[Task stateNamesArray]];
   [[taskView tableColumnWithIdentifier:@"state"] setDataCell:cell];
+  [[taskView tableColumnWithIdentifier:@"state"] setMaxWidth:128];
   [taskView setAutoresizesAllColumnsToFit:YES];
   /* 
    * FIXME : this shouldn't be needed but I can't make it
    * work by editing the interface with Gorm...
    * [[taskView superview] superview] is the ScrollView
+   * 
+   * Edit : I don't know if the bug comes from Gorm or
+   * NSTabView but here's an explanation : there is a NSView
+   * between NSTabView and each tab view (NSTabView -> NSView -> DayView,
+   * NSTabView -> NSView -> WeekView etc) and, except for the first tab,
+   * the intermediate view's autoresizingMask is 0
    */
   [[[[taskView superview] superview] superview] setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
+  [[weekView superview] setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
   [taskView setUsesAlternatingRowBackgroundColors:YES];
-  [[taskView tableColumnWithIdentifier:@"state"] setMaxWidth:92];
   [taskView setTarget:self];
   [taskView setDoubleAction:@selector(editAppointment:)];
+  [summary sizeLastColumnToFit];
   [summary setTarget:self];
   [summary setDoubleAction:@selector(editAppointment:)];
   [window setFrameAutosaveName:@"mainWindow"];
-  [[tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Day"]] setLabel:[[_selectedDay calendarDate] descriptionWithCalendarFormat:@"%e %b"]];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)not
 {
-  [self updateSummaryData];
-  [dayView reloadData];
+  NSWindow *win;
+  unsigned int width, height;
+
+  win = [NSApp iconWindow];
+  width = [[win contentView] bounds].size.width;
+  height = [[win contentView] bounds].size.height;  
+  _appicon = [[AppIcon alloc] initWithFrame: NSMakeRect(1, 1, width - 2, height - 2)];
+  [[win contentView] addSubview:_appicon];
+
+  [self registerForServices];
   [NSApp setServicesProvider: self];
-  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChanged object:nil];
+  /*
+   * We should register these notifications before allocating
+   * the StoreManager to get all data updates. To avoid
+   * numerous invisible updates which would slow the startup,
+   * register only when the application is ready.
+   */
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChangedInStoreManager object:nil];
+  /* FIXME : this is overkill, we should only refresh the views for visual changes */
+  [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SAStatusChangedForStore object:nil];
+  /* Set the selected day : this will update all views and titles (but not the summary */
+  [calendar setDate:[Date today]];
+  /*
+   * If stores are loaded before this is executed (it happens
+   * when the gui is super slow, with remote X for example)
+   * we miss the dataChanged notification (that's the purpose 
+   * of late registering after all) and the summary will be
+   * empty so this is needed here.
+   */
+  [self updateSummaryData];
+  /* This will init the alarms for all loaded elements needing one */
+  [AlarmManager globalManager];
 }
 
 - (void)applicationWillTerminate:(NSNotification*)aNotification
 {
+  [[ConfigManager globalConfig] unregisterClient:self];
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   [_summaryRoot release];
-  [_today release];
-  [_tomorrow release];
-  [_soon release];
-  [_results release];
   [_pc release];
   /* 
-   * Ugly workaround : [_sm release] should force the
-   * modified stores to synchronise their data but it 
-   * doesn't work. We're leaking a object reference.
+   * FIXME : we shouldn't have to release the store
+   * manager as we don't retain it. It's a global
+   * instance that should synchronise itself when
+   * it's freed
    */
-  [_sm synchronise];
   [_sm release];
-  [_editor release];
-  [_taskEditor release];
   RELEASE(_selectedDay);
 }
 
@@ -209,7 +307,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (int)_sensibleStartForDuration:(int)duration
 {
   int minute = [dayView firstHour] * 60;
-  NSEnumerator *enumerator = [[self scheduledAppointmentsForDay:nil] objectEnumerator];
+  NSEnumerator *enumerator = [[_sm scheduledAppointmentsForDay:_selectedDay] objectEnumerator];
   Event *apt;
 
   while ((apt = [enumerator nextObject])) {
@@ -224,21 +322,22 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (void)addAppointment:(id)sender
 {
-  Date *date = [[calendar date] copy];
+  Event *apt;
+  Date *date;
+
+  date = [[calendar date] copy];
+  [date setIsDate:NO];
   [date setMinute:[self _sensibleStartForDuration:60]];
-  Event *apt = [[Event alloc] initWithStartDate:date 
-					  duration:60
-					  title:@"edit title..."];
-  if (apt && [_editor editAppointment:apt withStoreManager:_sm])
-    [tabs selectTabViewItemWithIdentifier:@"Day"];
+  apt = [[Event alloc] initWithStartDate:date duration:60 title:_(@"edit title...")];
+  [AppointmentEditor editorForEvent:apt];
   [date release];
   [apt release];
 }
 
 - (void)addTask:(id)sender
 {
-  Task *task = [[Task alloc] initWithSummary:@"edit summary..."];
-  if (task && [_taskEditor editTask:task withStoreManager:_sm])
+  Task *task = [[Task alloc] initWithSummary:_(@"edit summary...")];
+  if (task && [TaskEditor editorForTask:task])
     [tabs selectTabViewItemWithIdentifier:@"Tasks"];
   [task release];
 }
@@ -261,45 +360,54 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   }
   task = [Task new];
   if ([aString length] > 40) {
-    [task setSummary:@"New note"];
+    [task setSummary:_(@"New task")];
     [task setText:AUTORELEASE([[NSAttributedString alloc ] initWithString:aString])];
   } else
     [task setSummary:aString];
-  if (task && [_taskEditor editTask:task withStoreManager:_sm])
+  if (task && [TaskEditor editorForTask:task])
     [tabs selectTabViewItemWithIdentifier:@"Tasks"];
   [task release];
 }
 
 - (void)editAppointment:(id)sender
 {
-  if (_clickedElement) {
-    if ([_clickedElement isKindOfClass:[Event class]])
-      [_editor editAppointment:(Event *)_clickedElement withStoreManager:_sm];
-    else if ([_clickedElement isKindOfClass:[Task class]])
-      [_taskEditor editTask:(Task *)_clickedElement withStoreManager:_sm];
+  id lastSelection = [selectionManager lastObject];
+
+  if (lastSelection) {
+    if ([lastSelection isKindOfClass:[Event class]])
+      [AppointmentEditor editorForEvent:(Event *)lastSelection];
+    else if ([lastSelection isKindOfClass:[Task class]])
+      [TaskEditor editorForTask:(Task *)lastSelection];
+    else
+      NSLog(@"We should never come here...");
   }
 }
 
 - (void)delAppointment:(id)sender
 {
-  if (_clickedElement) {
-    [[_clickedElement store] remove:_clickedElement];
-    _clickedElement = nil;
-  }
+  NSEnumerator *enumerator = [selectionManager enumerator];
+  Element *el;
+
+  while ((el = [enumerator nextObject]))
+    [[el store] remove:el];
+  [selectionManager clear];
 }
 
 - (void)exportAppointment:(id)sender;
 {
+  NSEnumerator *enumerator = [selectionManager enumerator];
   NSSavePanel *panel = [NSSavePanel savePanel];
   NSString *str;
   iCalTree *tree;
+  Element *el;
 
-  if (_clickedElement) {
+  if ([selectionManager count] > 0) {
     [panel setRequiredFileType:@"ics"];
-    [panel setTitle:@"Export as"];
-    if ([panel runModalForDirectory:nil file:[_clickedElement summary]] == NSOKButton) {
+    [panel setTitle:_(@"Export as")];
+    if ([panel runModalForDirectory:nil file:[[selectionManager lastObject] summary]] == NSOKButton) {
       tree = [iCalTree new];
-      [tree add:_clickedElement];
+      while ((el = [enumerator nextObject]))
+	[tree add:el];
       str = [tree iCalTreeAsString];
       if (![str writeToFile:[panel filename] atomically:NO])
 	NSLog(@"Unable to write to file %@", [panel filename]);
@@ -320,36 +428,73 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (void)copy:(id)sender
 {
-  _selection = (Event *)_clickedElement;
-  _deleteSelection = NO;
+  [selectionManager copySelection];
 }
 
 - (void)cut:(id)sender
 {
-  _selection = (Event *)_clickedElement;
-  _deleteSelection = YES;
+  [selectionManager cutSelection];
 }
 
 - (void)paste:(id)sender
 {
-  if (_selection) {
+  if ([selectionManager copiedCount] > 0) {
+    NSEnumerator *enumerator = [[selectionManager paste] objectEnumerator];
     Date *date = [[calendar date] copy];
-    if (_deleteSelection) {
-      [date setMinute:[self _sensibleStartForDuration:[_selection duration]]];
-      [_selection setStartDate:date];
-      [[_selection store] update:_selection];
-      _selection = nil;
-    } else {
-      Event *new = [_selection copy];
-      [date setMinute:[self _sensibleStartForDuration:[new duration]]];
-      [new setStartDate:date];
-      [[_selection store] add:new];
-      [new release];
+    Event *el;
+    id <MemoryStore> store;
+    int start;
+
+    [date setIsDate:NO];
+    while ((el = [enumerator nextObject])) {
+      /* FIXME : store property could be handled by Event:copy ? */
+      store = [el store];
+      start = [[el startDate] minuteOfDay];
+      if ([selectionManager lastOperation] == SMCopy)
+	el = [el copy];
+      [date setMinute:start];
+      [el setStartDate:date];
+      if ([selectionManager lastOperation] == SMCopy) {
+	[store add:el];
+	/*
+	 * FIXME : the new event is now in store's dictionary, we 
+	 * should be able to release it. If we do, the application 
+	 * crashes when we delete this event, trying to release it
+	 * one time too many. I can't find the bug
+	 * [el release];
+	 */
+      } else {
+	[store update:el];
+      }     
     }
     [date release];
   }
 }
 
+- (void)today:(id)sender
+{
+  [calendar setDate:[Date today]];
+}
+- (void)nextDay:(id)sender
+{
+  [calendar setDate:[Date dateWithTimeInterval:86400 sinceDate:[calendar date]]];
+}
+
+- (void)previousDay:(id)sender
+{
+  [calendar setDate:[Date dateWithTimeInterval:-86400 sinceDate:[calendar date]]];
+}
+
+- (void)nextWeek:(id)sender
+{
+  [calendar setDate:[Date dateWithTimeInterval:86400*7 sinceDate:[calendar date]]];
+}
+
+- (void)previousWeek:(id)sender
+{
+  [calendar setDate:[Date dateWithTimeInterval:86400*-7 sinceDate:[calendar date]]];
+}
+
 - (void)performSearch
 {
   NSEnumerator *enumerator;
@@ -362,10 +507,11 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
       if ([event contains:[search stringValue]])
 	[_results addChild:[DataTree dataTreeWithAttributes:[self attributesFrom:event and:[event startDate]]]];
     }
-    [_results setValue:[NSString stringWithFormat:@"%d item(s)", [[_results children] count]] forKey:@"details"];;
     [_results sortChildrenUsingFunction:compareDataTreeElements context:nil];
     [summary expandItem:_results];
-  }
+    [_results setValue:[NSString stringWithFormat:_(@"Search results (%d items)"), [[_results children] count]] forKey:@"title"];
+  } else
+    [_results setValue:_(@"Search results") forKey:@"title"];
 }
 
 - (void)doSearch:(id)sender
@@ -378,51 +524,38 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (void)clearSearch:(id)sender
 {
   [search setStringValue:@""];
-  [_results removeChildren];
-  [_results setValue:@"" forKey:@"details"];;
+  [self performSearch];
   [summary reloadData];
   [window makeFirstResponder:search];
 }
 
 - (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
 {
-  BOOL itemSelected = _clickedElement != nil;
   SEL action = [menuItem action];
 
   if (sel_eq(action, @selector(copy:)))
-    return itemSelected && [_clickedElement isKindOfClass:[Event class]] && [[_clickedElement store] writable];
+    return [selectionManager count] > 0;
   if (sel_eq(action, @selector(cut:)))
-    return itemSelected && [_clickedElement isKindOfClass:[Event class]] && [[_clickedElement store] writable];
+    return [selectionManager count] > 0;
   if (sel_eq(action, @selector(editAppointment:)))
-    return itemSelected;
+    return [selectionManager count] == 1;
   if (sel_eq(action, @selector(delAppointment:)))
-    return itemSelected;
+    return [selectionManager count] > 0;
   if (sel_eq(action, @selector(exportAppointment:)))
-    return itemSelected;
+    return [selectionManager count] > 0;
   if (sel_eq(action, @selector(paste:)))
-    return _selection != nil && [[_selection store] writable];
+    return [selectionManager copiedCount] > 0;
   return YES;
 }
 
-/* DayViewDataSource protocol */
-- (NSSet *)scheduledAppointmentsForDay:(Date *)day
-{
-  NSMutableSet *dayEvents = [NSMutableSet setWithCapacity:8];
-  NSEnumerator *enumerator = [[_sm allEvents] objectEnumerator];
-  Event *event;
-
-  if (day == nil)
-    day = _selectedDay;
-  while ((event = [enumerator nextObject]))
-    if ([event isScheduledForDay:day])
-      [dayEvents addObject:event];
-  return dayEvents;
-}
-
 - (void)dataChanged:(NSNotification *)not
 {
-  _clickedElement = nil;
+  /* 
+   * FIXME : if a selected event was deleted by another application, 
+   * the selection will reference a non existing object
+   */
   [dayView reloadData];
+  [weekView reloadData];
   [taskView reloadData];
   [self performSearch];
   [self updateSummaryData];
@@ -430,23 +563,26 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 
 - (id)validRequestorForSendType:(NSString *)sendType returnType:(NSString *)returnType
 {
-  if (_clickedElement && (!sendType || [sendType isEqual:NSFilenamesPboardType] || [sendType isEqual:NSStringPboardType]))
+  if ([selectionManager count] && (!sendType || [sendType isEqual:NSFilenamesPboardType] || [sendType isEqual:NSStringPboardType]))
     return self;
   return nil;
 }
 - (BOOL)writeSelectionToPasteboard:(NSPasteboard *)pboard types:(NSArray *)types
 {
+  NSEnumerator *enumerator = [selectionManager enumerator];
+  Element *el;
   NSString *ical;
   NSString *filename;
   iCalTree *tree;
   NSFileWrapper *fw;
   BOOL written;
 
-  if (!_clickedElement)
+  if ([selectionManager count] == 0)
     return NO;
   NSAssert([types count] == 1, @"It seems our assumption was wrong");
   tree = AUTORELEASE([iCalTree new]);
-  [tree add:_clickedElement];
+  while ((el = [enumerator nextObject]))
+    [tree add:el];
   ical = [tree iCalTreeAsString];
 
   if ([types containsObject:NSFilenamesPboardType]) {
@@ -455,7 +591,7 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
       NSLog(@"Unable to encode into NSFileWrapper");
       return NO;
     }
-    filename = [NSString stringWithFormat:@"%@/%@.ics", NSTemporaryDirectory(), [_clickedElement summary]];
+    filename = [NSString stringWithFormat:@"%@/%@.ics", NSTemporaryDirectory(), [[selectionManager lastObject] summary]];
     written = [fw writeToFile:filename atomically:YES updateFilenames:YES];
     [fw release];
     if (!written) {
@@ -471,6 +607,11 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
   }
   return NO;
 }
+
+- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key
+{
+  [_appicon setNeedsDisplay:YES];
+}
 @end
 
 @implementation AppController(NSOutlineViewDataSource)
@@ -482,6 +623,8 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 }
 - (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
 {
+  if (item == nil)
+    return YES;
   return [[item children] count] > 0;
 }
 - (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
@@ -500,19 +643,23 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
 {
   id object = [item valueForKey:@"object"];
-  NSString *tabIdentifier = [[tabs selectedTabViewItem] identifier];;
+  NSString *tabIdentifier = [[tabs selectedTabViewItem] identifier];
+  Date *date;
 
   if (object && [object isKindOfClass:[Event class]]) {
-    [calendar setDate:[item valueForKey:@"date"]];
-    if (![tabIdentifier isEqualToString:@"Day"])
+    date = [[item valueForKey:@"date"] copy];
+    [date setIsDate:YES];
+    [calendar setDate:date];
+    [date release];
+    if (![tabIdentifier isEqualToString:@"Day"] && ![tabIdentifier isEqualToString:@"Week"])
       [tabs selectTabViewItemWithIdentifier:@"Day"];
-    _clickedElement = object;
+    [selectionManager set:object];
     return YES;
   }
   if (object && [object isKindOfClass:[Task class]]) {
     if (![tabIdentifier isEqualToString:@"Tasks"])
       [tabs selectTabViewItemWithIdentifier:@"Tasks"];
-    _clickedElement = object;
+    [selectionManager set:object];
     return YES;
   }
   return NO;
@@ -523,17 +670,25 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 - (void)calendarView:(CalendarView *)cs selectedDateChanged:(Date *)date
 {
   NSTabViewItem *dayTab = [tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Day"]];
-  _clickedElement = nil;
+  NSTabViewItem *weekTab = [tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Week"]];
+  NSTabViewItem *taskTab = [tabs tabViewItemAtIndex:[tabs indexOfTabViewItemWithIdentifier:@"Tasks"]];
+
   ASSIGNCOPY(_selectedDay, date);
-  [dayView reloadData];
-  [dayTab setLabel:[[date calendarDate] descriptionWithCalendarFormat:@"%e %b"]];
-  /* FIXME : this, somehow, prevents the selected month to show up in the popup */
-  /* [tabs selectTabViewItem:dayTab]; */
+  [dayView setDate:date];
+  [weekView setDate:date];
+  /* Hack to enable translation of this tab's label */
+  [taskTab setLabel:_(@"Tasks")];
+  [dayTab setLabel:[[_selectedDay calendarDate] descriptionWithCalendarFormat:@"%e %b"]];
+  [weekTab setLabel:[NSString stringWithFormat:_(@"Week %d"), [_selectedDay weekOfYear]]];
+  if ([tabs selectedTabViewItem] != dayTab && [tabs selectedTabViewItem] != weekTab)
+    [tabs selectTabViewItem:dayTab];
   [tabs setNeedsDisplay:YES];
+  [self setWindowTitle];
 }
 - (void)calendarView:(CalendarView *)cs currentDateChanged:(Date *)date
 {
   [self updateSummaryData];
+  [_appicon setNeedsDisplay:YES];
 }
 - (void)calendarView:(CalendarView *)cs userActionForDate:(Date *)date
 {
@@ -541,31 +696,34 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 }
 @end
 
-@implementation AppController(DayViewDelegate)
-- (void)dayView:(DayView *)dayview editEvent:(Event *)event;
+@implementation AppController(AppointmentViewDelegate)
+- (void)viewEditEvent:(Event *)event;
 {
-  _clickedElement = event;
-  [_editor editAppointment:event withStoreManager:_sm];
+  [AppointmentEditor editorForEvent:event];
 }
-- (void)dayView:(DayView *)dayview modifyEvent:(Event *)event
+- (void)viewModifyEvent:(Event *)event
 {
   [[event store] update:event];
 }
-- (void)dayView:(DayView *)dayview createEventFrom:(int)start to:(int)end
+- (void)viewCreateEventFrom:(int)start to:(int)end
 {
   Date *date = [[calendar date] copy];
+  [date setIsDate:NO];
   [date setMinute:start];
   Event *apt = [[Event alloc] initWithStartDate:date 
-			      duration:end - start 
-			      title:@"edit title..."];
+			               duration:end - start 
+			                  title:_(@"edit title...")];
   if (apt)
-    [_editor editAppointment:apt withStoreManager:_sm];
+    [AppointmentEditor editorForEvent:apt];
   [date release];
   [apt release];
 }
-- (void)dayView:(DayView *)dayview selectEvent:(Event *)event
+- (void)viewSelectEvent:(Event *)event
 {
-  _clickedElement = event;
+}
+- (void)viewSelectDate:(Date *)date
+{
+  [calendar setDate:date];
 }
 @end
 
@@ -574,10 +732,6 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
   return [[_sm allTasks] count];
 }
-- (BOOL)tableView:(NSTableView *)tableView acceptDrop:(id <NSDraggingInfo>)info row:(int)row dropOperation:(NSTableViewDropOperation)operation
-{
-  return NO;
-}
 - (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
 {
   Task *task = [[_sm allTasks] objectAtIndex:rowIndex];
@@ -608,18 +762,13 @@ NSComparisonResult compareDataTreeElements(id a, id b, void *context)
 {
   int index = [taskView selectedRow];
   if (index > -1)
-    _clickedElement = [[_sm allTasks] objectAtIndex:index];
+    [selectionManager set:[[_sm allTasks] objectAtIndex:index]];
 }
 @end
 
-/* FIXME : gross hack around selection problem */
 @implementation AppController(NSTabViewDelegate)
 - (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
 {
-  if ([[tabViewItem identifier] isEqualToString:@"Day"])
-    [taskView deselectAll:self];
-  if ([[tabViewItem identifier] isEqualToString:@"Tasks"])
-    [dayView deselectAll:self];
-  _clickedElement = nil;
+  [self setWindowTitle];
 }
 @end
