@@ -17,10 +17,13 @@
   id allDay;
   id ok;
   id until;
+  id time;
+  id timeText;
   Date *startDate;
+  Event *_event;
 }
 
-- (BOOL)editAppointment:(Event *)data withStoreManager:(StoreManager *)sm;
++ (AppointmentEditor *)editorForEvent:(Event *)event;
 - (void)validate:(id)sender;
 - (void)cancel:(id)sender;
 - (void)selectFrequency:(id)sender;
