@@ -1,21 +1,20 @@
 /* emacs objective-c mode -*- objc -*- */
 
 #import <AppKit/AppKit.h>
-#import "StoreManager.h"
 
 @interface CalendarView : NSView
 {
   NSTextField *title;
+  NSButton *obl;
+  NSButton *tbl;
+  NSButton *obr;
+  NSButton *tbr;
   Date *date;
   Date *monthDisplayed;
-  NSPopUpButton *month;
-  NSStepper *stepper;
-  NSTextField *text;
   NSMatrix *matrix;
   NSFont *normalFont;
   NSFont *boldFont;
   IBOutlet id delegate;
-  IBOutlet NSObject <AgendaDataSource> *dataSource;
   NSTimer *_dayTimer;
   int bezeledCell;
 }
@@ -24,10 +23,8 @@
 - (Date *)date;
 - (NSString *)dateAsString;
 - (id)delegate;
-- (id)dataSource;
 - (void)setDate:(Date *)date;
 - (void)setDelegate:(id)delegate;
-- (void)setDataSource:(NSObject <AgendaDataSource> *)source;
 @end
 
 @interface NSObject(CalendarViewDelegate)
