@@ -21,7 +21,26 @@
 }
 @end
 
+
 @implementation CalendarView
+static NSImage *_1left;
+static NSImage *_2left;
+static NSImage *_1right;
+static NSImage *_2right;
++ (void)initialize
+{
+  NSString *path;
+
+  path = [[NSBundle mainBundle] pathForImageResource:@"1left"];
+  _1left = [[NSImage alloc] initWithContentsOfFile:path];
+  path = [[NSBundle mainBundle] pathForImageResource:@"2left"];
+  _2left = [[NSImage alloc] initWithContentsOfFile:path];
+  path = [[NSBundle mainBundle] pathForImageResource:@"1right"];
+  _1right = [[NSImage alloc] initWithContentsOfFile:path];
+  path = [[NSBundle mainBundle] pathForImageResource:@"2right"];
+  _2right = [[NSImage alloc] initWithContentsOfFile:path];
+}
+
 - (void)dealloc
 {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
@@ -30,13 +49,14 @@
   [_dayTimer invalidate];
   RELEASE(_dayTimer);
   RELEASE(delegate);
-  RELEASE(dataSource);
   [boldFont release];
   [normalFont release];
   [title release];
   [matrix release];
-  [month release];
-  [stepper release];
+  [obl release];
+  [tbl release];
+  [obr release];
+  [tbr release];
   [super dealloc];
 }
 
@@ -50,14 +70,12 @@
 
   self = [super initWithFrame:frame];
   if (self) {
-    NSArray *months = [[NSUserDefaults standardUserDefaults] objectForKey:NSMonthNameArray];
     NSArray *days = [[NSUserDefaults standardUserDefaults] objectForKey:NSShortWeekDayNameArray];
     boldFont = RETAIN([NSFont boldSystemFontOfSize:11]);
     normalFont = RETAIN([NSFont systemFontOfSize:11]);
     delegate = nil;
-    dataSource = nil;
 
-    title = [[NSTextField alloc] initWithFrame: NSMakeRect(85, 142, 80, 20)];
+    title = [[NSTextField alloc] initWithFrame: NSMakeRect(32, 128, 168, 20)];
     [title setEditable:NO];
     [title setDrawsBackground:NO];
     [title setBezeled:NO];
@@ -67,26 +85,31 @@
     [title setAlignment: NSCenterTextAlignment];
     [self addSubview: title];
 
-    month = [[NSPopUpButton alloc] initWithFrame: NSMakeRect(8, 140, 80, 25)];
-    [month setFont:normalFont];
-    [month addItemsWithTitles: months];
-    [month setTarget: self];
-    [month setAction: @selector(selectMonth:)];
-    [self addSubview: month];
-
-    text = [[NSTextField alloc] initWithFrame: NSMakeRect(164, 142, 40, 21)];
-    [text setEditable: NO];
-    [text setAlignment: NSRightTextAlignment];
-    [text setFont:normalFont];
-    [self addSubview: text];
-
-    stepper = [[NSStepper alloc] initWithFrame: NSMakeRect(206, 140, 16, 25)];
-    [stepper setMinValue: 1970];
-    [stepper setMaxValue: 2037];
-    [stepper setTarget: self];
-    [stepper setAction: @selector(selectYear:)];
-    [stepper setFont:normalFont];
-    [self addSubview: stepper];
+    tbl = [[NSButton alloc] initWithFrame:NSMakeRect(9, 128, 12, 20)];
+    [tbl setImage:_2left];
+    [tbl setBordered:NO];
+    [tbl setTarget:self];
+    [tbl setAction:@selector(previousYear:)];
+    [self addSubview:tbl];
+    obl = [[NSButton alloc] initWithFrame:NSMakeRect(22, 128, 12, 20)];
+    [obl setImage:_1left];
+    [obl setBordered:NO];
+    [obl setTarget:self];
+    [obl setAction:@selector(previousMonth:)];
+    [self addSubview:obl];
+    obr = [[NSButton alloc] initWithFrame:NSMakeRect(201, 128, 12, 20)];
+    [obr setButtonType:NSMomentaryPushInButton];
+    [obr setImage:_1right];
+    [obr setBordered:NO];
+    [obr setTarget:self];
+    [obr setAction:@selector(nextMonth:)];
+    [self addSubview:obr];
+    tbr = [[NSButton alloc] initWithFrame:NSMakeRect(214, 128, 12, 20)];
+    [tbr setImage:_2right];
+    [tbr setBordered:NO];
+    [tbr setTarget:self];
+    [tbr setAction:@selector(nextYear:)];
+    [self addSubview:tbr];
 
     NSTextFieldCell *cell = [NSTextFieldCell new];
     [cell setEditable: NO];
@@ -94,7 +117,7 @@
     [cell setAlignment: NSRightTextAlignment];
     [cell setFont:normalFont];
 
-    matrix = [[NSMatrix alloc] initWithFrame: NSMakeRect(9, 8, 220, 128)
+    matrix = [[NSMatrix alloc] initWithFrame: NSMakeRect(9, 6, 220, 128)
 			       mode: NSListModeMatrix
 			       prototype: cell
 			       numberOfRows: 7
@@ -132,22 +155,18 @@
     [formatter release];
     [self addSubview: matrix];
 
-    /*
-     * FIXME : this should be [Date today] but it leads to
-     * every appointment starting at 00:00. Probably a problem 
-     * between ical time and ical date
-     */
-    now = [Date now];
+    now = [Date today];
     [self setDate:now];
     [now incrementDay];
-    [now clearTime];
     _dayTimer = [[NSTimer alloc] initWithFireDate:[now calendarDate]
-				 interval:86400 target:self 
-				 selector:@selector(dayChanged:) 
-				 userInfo:nil 
-				 repeats:YES];
+				         interval:86400
+				           target:self
+				         selector:@selector(dayChanged:)
+				         userInfo:nil
+				          repeats:YES];
     [[NSRunLoop currentRunLoop] addTimer:_dayTimer forMode:NSDefaultRunLoopMode];
-    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChanged object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SADataChangedInStoreManager object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(dataChanged:) name:SAStatusChangedForStore object:nil];
   }
   return self;
 }
@@ -190,17 +209,16 @@
   NSColor *clear = [NSColor clearColor];
   NSColor *white = [NSColor whiteColor];
   NSColor *black = [NSColor blackColor];
+  StoreManager *sm = [StoreManager globalManager];
 
   [self clearSelectedDay];
   today = [Date today];
   day = [monthDisplayed copy];
   [day setDay: 1];
-  week = [day weekOfYear];
   column = [day weekday];
-  if (!column)
-    column = 7;
   [day changeDayBy:1-column];
-  for (row = 1; row < 7; row++, week++) {
+  for (row = 1; row < 7; row++) {
+    week = [day weekOfYear];
     [[matrix cellAtRow:row column:0] setStringValue:[NSString stringWithFormat:@"%d ", week]];
     for (column = 1; column < 8; column++, [day incrementDay]) {
       cell = [matrix cellAtRow: row column: column];
@@ -211,8 +229,8 @@
 	[cell setBackgroundColor:clear];
 	[cell setDrawsBackground:NO];
       }
-      [cell setObjectValue:[day copy]];
-      if (dataSource && [[dataSource scheduledAppointmentsForDay:day] count])
+      [cell setObjectValue:AUTORELEASE([day copy])];
+      if ([[sm scheduledAppointmentsForDay:day] count])
 	[cell setFont:boldFont];
       else
 	[cell setFont: normalFont];
@@ -233,30 +251,39 @@
     [delegate calendarView:self currentDateChanged:[Date today]];
 }
 
-- (void)selectMonth:(id)sender
+- (void)previousYear:(id)sender
 {
-  int idx = [month indexOfSelectedItem] + 1;
-  [date setMonth: idx];
-  [monthDisplayed setMonth: idx];
-  [self updateView];
-  if ([delegate respondsToSelector:@selector(calendarView:selectedDateChanged:)])
-    [delegate calendarView:self selectedDateChanged:date];
-}
+  Date *cdate = AUTORELEASE([date copy]);
 
-- (void)selectYear:(id)sender
+  [cdate setYear:[cdate year]-1];
+  [self setDate:cdate];
+}
+- (void)previousMonth:(id)sender
 {
-  int year = [stepper intValue];
-  [text setIntValue: year];
-  [date setYear: year];
-  [monthDisplayed setYear: year];
-  [self updateView];
-  if ([delegate respondsToSelector:@selector(calendarView:selectedDateChanged:)])
-    [delegate calendarView:self selectedDateChanged:date];
+  Date *cdate = AUTORELEASE([date copy]);
+
+  [cdate setMonth:[cdate monthOfYear]-1];
+  [self setDate:cdate];
+}
+- (void)nextMonth:(id)sender
+{
+  Date *cdate = AUTORELEASE([date copy]);
+
+  [cdate setMonth:[cdate monthOfYear]+1];
+  [self setDate:cdate];
+}
+- (void)nextYear:(id)sender
+{
+  Date *cdate = AUTORELEASE([date copy]);
+
+  [cdate setYear:[cdate year]+1];
+  [self setDate:cdate];
 }
 
 - (void)selectDay:(id)sender
 {
   id day = [[matrix selectedCell] objectValue];
+
   if ([day isKindOfClass:[Date class]]) {
     [self clearSelectedDay];
     ASSIGNCOPY(date, day);
@@ -272,7 +299,7 @@
     [delegate calendarView:self userActionForDate:date];
 }
 
-- (void)setDelegate: (id)aDelegate
+- (void)setDelegate:(id)aDelegate
 {
   ASSIGN(delegate, aDelegate);
 }
@@ -283,11 +310,9 @@
 
 - (void)setDate:(Date *)nDate
 {
+  NSAssert([nDate isDate], @"Calender expects a date");
   ASSIGNCOPY(date, nDate);
   ASSIGNCOPY(monthDisplayed, nDate);
-  [text setIntValue: [nDate year]];
-  [stepper setIntValue: [nDate year]];
-  [month selectItemAtIndex: [nDate monthOfYear] - 1];
   [self updateView];
   if ([delegate respondsToSelector:@selector(calendarView:selectedDateChanged:)])
     [delegate calendarView:self selectedDateChanged:date];
@@ -299,17 +324,7 @@
 
 - (NSString *)dateAsString
 {
-  return [[date calendarDate] descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSShortDateFormatString]];
-}
-
-- (void)setDataSource:(NSObject <AgendaDataSource> *)source
-{
-  ASSIGN(dataSource, source);
-  [self updateView];
-}
-- (id)dataSource
-{
-  return dataSource;
+  return [[date calendarDate] descriptionWithCalendarFormat:[[NSUserDefaults standardUserDefaults] objectForKey:NSDateFormatString]];
 }
 
 - (void)dataChanged:(NSNotification *)not
