@@ -1,9 +1,10 @@
 /* emacs buffer mode hint -*- objc -*- */
 
 @class ConfigManager;
+@class NSColor;
 
 @protocol ConfigListener
-- (void)config:(ConfigManager*)config dataDidChangedForKey:(NSString *)key;
+- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key;
 @end
 
 @interface ConfigManager : NSObject
@@ -12,14 +13,13 @@
   ConfigManager *_parent;
   NSMutableDictionary *_defaults;
   NSMutableDictionary *_dict;
-  NSMutableDictionary *_cpkey;
 }
 
 - (ConfigManager *)initForKey:(NSString *)key withParent:(ConfigManager *)parent;
 + (ConfigManager *)globalConfig;
 - (void)registerDefaults:(NSDictionary *)defaults;
 - (void)registerClient:(id <ConfigListener>)client forKey:(NSString *)key;
-- (void)unregisterClient:(id <ConfigListener>)client forKey:(NSString*)key;
+- (void)unregisterClient:(id <ConfigListener>)client forKey:(NSString *)key;
 - (void)unregisterClient:(id <ConfigListener>)client;
 - (id)objectForKey:(NSString *)key;
 - (void)removeObjectForKey:(NSString *)key;
@@ -28,6 +28,6 @@
 - (void)setInteger:(int)value forKey:(NSString *)key;
 - (NSDictionary *)dictionaryForKey:(NSString *)key;
 - (void)setDictionary:(NSDictionary *)dict forKey:(NSString *)key;
-
+- (NSColor *)colorForKey:(NSString *)key;
+- (void)setColor:(NSColor *)value forKey:(NSString *)key;
 @end
-
