@@ -1,13 +1,14 @@
 #import <Foundation/Foundation.h>
 #import "ConfigManager.h"
 
-@implementation ConfigManager(Private)
+static ConfigManager *singleton;
+static NSMutableDictionary *_cpkey;
 
+@implementation ConfigManager(Private)
 - (ConfigManager *)initRoot
 {
   self = [super init];
   if (self) {
-    _cpkey = [NSMutableDictionary new];
     _dict = [NSMutableDictionary new];
     _defaults = [NSMutableDictionary new];
     [_dict setDictionary:[[NSUserDefaults standardUserDefaults] 
@@ -15,20 +16,22 @@
   }
   return self;
 }
-
 @end
 
-
 @implementation ConfigManager
-
-static ConfigManager *singleton;
++ (void)initialize
+{
+  if ([ConfigManager class] == self) {
+    _cpkey = [NSMutableDictionary new];
+    singleton = [[ConfigManager alloc] initRoot];
+  }
+}
 
 - (ConfigManager *)initForKey:(NSString *)key withParent:(ConfigManager *)parent
 {
   NSAssert(key != nil, @"ConfigManager initForKey called with nil key");
   self = [super init];
   if (self) {
-    _cpkey = [NSMutableDictionary new];
     _dict = [NSMutableDictionary new];
     _defaults = [NSMutableDictionary new];
     if (parent == nil)
@@ -42,8 +45,6 @@ static ConfigManager *singleton;
 
 + (ConfigManager *)globalConfig
 {
-  if (singleton == nil)
-    singleton = [[ConfigManager alloc] initRoot];
   return singleton;
 }
 
@@ -53,28 +54,21 @@ static ConfigManager *singleton;
   RELEASE(_parent);
   [_defaults release];
   [_dict release];
-  [_cpkey release];
   [super dealloc];
 }
 
 - (void)notifyListenerForKey:(NSString *)key
 {
   NSEnumerator *enumerator;
-  NSMutableSet *set, *tmp;
   id <ConfigListener> listener;
-  if (key)
-    set = [_cpkey objectForKey:key];
-  else {
-    set = [NSMutableSet new];
-    enumerator = [_cpkey objectEnumerator];
-    while ((tmp = [enumerator nextObject]))
-      [set unionSet:tmp];
-  }
-  enumerator = [set objectEnumerator];
-  while ((listener = [enumerator nextObject]))
+  NSValue *value;
+
+  NSAssert(key != nil, @"Missing key");
+  enumerator = [[_cpkey objectForKey:key] objectEnumerator];
+  while ((value = [enumerator nextObject])) {
+    listener = [value nonretainedObjectValue];
     [listener config:self dataDidChangedForKey:key];
-  if (!key)
-    [set release];
+  }
 }
 
 - (void)registerDefaults:(NSDictionary *)defaults
@@ -141,31 +135,65 @@ static ConfigManager *singleton;
   [self setObject:dict forKey:key];
 }
 
+- (NSColor *)colorForKey:(NSString *)key
+{
+  id obj = [self objectForKey:key];
+
+  if ([obj isKindOfClass:[NSData class]])
+    return [NSUnarchiver unarchiveObjectWithData:obj];
+  return [NSColor colorFromString:obj];
+}
+
+- (void)setColor:(NSColor *)value forKey:(NSString *)key
+{
+  [self setObject:[value description] forKey:key];
+}
+
 - (void)registerClient:(id <ConfigListener>)client forKey:(NSString*)key
 {
   NSAssert(key != nil, @"You have to register for a specific key");
   NSMutableSet *listeners = [_cpkey objectForKey:key];
+
   if (listeners)
-    [listeners addObject:client];
+    [listeners addObject:[NSValue valueWithNonretainedObject:client]];
   else
-    listeners = [NSMutableSet setWithObject:client];
+    listeners = [NSMutableSet setWithObject:[NSValue valueWithNonretainedObject:client]];
   [_cpkey setObject:listeners forKey:key];
+ 
 }
 
 - (void)unregisterClient:(id <ConfigListener>)client forKey:(NSString*)key
 {
   NSMutableSet *set = [_cpkey objectForKey:key];
-  if (set)
-    [set removeObject:client];
+  NSSet *copy;
+  NSEnumerator *enumerator;
+  NSValue *v;
+
+  if (set) {
+    copy = [NSSet setWithSet:set];
+    enumerator = [copy objectEnumerator];
+    while ((v = [enumerator nextObject])) {
+      if (client == [v nonretainedObjectValue])
+	[set removeObject:v];
+    }
+  }
 }
 
 - (void)unregisterClient:(id <ConfigListener>)client
 {
   NSMutableSet *set;
+  NSSet *copy;
+  NSValue *v;
   NSEnumerator *enumerator = [_cpkey objectEnumerator];
+  NSEnumerator *en;
 
-  while ((set = [enumerator nextObject]))
-    [set removeObject:client];
+  while ((set = [enumerator nextObject])) {
+    copy = [NSSet setWithSet:set];
+    en = [copy objectEnumerator];
+    while ((v = [en nextObject])) {
+      if (client == [v nonretainedObjectValue])
+	[set removeObject:v];
+    }
+  }
 }
-
 @end
