@@ -1,7 +1,7 @@
 /* emacs buffer mode hint -*- objc -*- */
 
+#import "config.h"
 #import <Foundation/Foundation.h>
-#import <ical.h>
 
 @interface Date : NSObject
 {
@@ -10,7 +10,8 @@
 
 + (id)now;
 + (id)today;
-- (id)initWithCalendarDate:(NSCalendarDate *)cd withTime:(BOOL)time;
++ (id)dateWithTimeInterval:(NSTimeInterval)seconds sinceDate:(Date *)refDate;
++ (id)dateWithCalendarDate:(NSCalendarDate *)cd withTime:(BOOL)time;
 - (NSCalendarDate *)calendarDate;
 - (NSComparisonResult)compare:(id)aDate;
 - (NSComparisonResult)compareTime:(id)aTime;
@@ -24,15 +25,6 @@
 - (int)weekday;
 - (int)weekOfYear;
 - (int)numberOfDaysInMonth;
-- (int)daysUntil:(Date *)date;
-- (int)daysSince:(Date *)date;
-- (int)weeksUntil:(Date *)date;
-- (int)weeksSince:(Date *)date;
-- (int)monthsUntil:(Date *)date;
-- (int)monthsSince:(Date *)date;
-- (int)yearsUntil:(Date *)date;
-- (int)yearSince:(Date *)date;
-- (void)clearTime;
 - (void)setSecondOfMinute:(int)second;
 - (void)setMinute:(int)minute;
 - (void)setDay:(int)day;
@@ -44,12 +36,12 @@
 - (void)changeMinuteBy:(int)diff;
 - (NSEnumerator *)enumeratorTo:(Date *)end;
 - (BOOL)isDate;
-- (void)setDate:(BOOL)date;
+- (void)setIsDate:(BOOL)date;
+- (NSTimeInterval)timeIntervalSince1970;
+- (NSTimeInterval)timeIntervalSinceDate:(Date *)anotherDate;
 @end
 
 @interface Date(iCalendar)
 - (id)initWithICalTime:(struct icaltimetype)time;
-- (void)setDateToICalTime:(struct icaltimetype)time;
 - (struct icaltimetype)iCalTime;
 @end
-
