@@ -14,6 +14,8 @@
   if (self != nil) {
     _start = [start copy];
     _end = [end copy];
+    [_start setIsDate:YES];
+    [_end setIsDate:YES];
     [_start changeDayBy:-1];
   }
   return self;
@@ -21,7 +23,7 @@
 - (id)nextObject
 {
   [_start incrementDay];
-  if ([_start daysUntil:_end] < 0)
+  if ([_end timeIntervalSinceDate:_start] < 0)
     return nil;
   return _start;
 }
@@ -50,6 +52,23 @@
  */
 
 @implementation Date
+static icaltimezone *gl_tz = NULL;
+static icaltimezone *gl_utc = NULL;
+static NSTimeZone *gl_nstz = nil;
++ (void)initialize
+{
+  const char *tzone;
+
+  if (!gl_tz) {
+    gl_nstz = RETAIN([[NSCalendarDate calendarDate] timeZone]);
+    tzone = [[gl_nstz description] cString];
+    gl_utc = icaltimezone_get_utc_timezone();
+    gl_tz = icaltimezone_get_builtin_timezone(tzone);
+    if (!gl_tz)
+      NSLog(@"Couldn't get a timezone corresponding to %s", tzone);
+  }
+}
+
 - (id)copyWithZone:(NSZone *)zone
 {
   Date *new = [Date allocWithZone:zone];
@@ -62,9 +81,33 @@
   return icaltime_compare_date_only(_time, ((Date *)aDate)->_time);
 }
 
+- (BOOL)isEqual:(id)aDate
+{
+  return icaltime_as_timet(_time) == icaltime_as_timet(((Date *)aDate)->_time);
+}
+
+- (NSUInteger)hash
+{
+  return icaltime_as_timet(_time);
+}
+/*
+ * This is a bit convoluted because icaltime_compare
+ * creates two dates in utc timezone to do the comparison.
+ * The conversion to utc means a datetime will be modified
+ * (up or down, depending on the local time zone) and a
+ * date won't.
+ */
 - (NSComparisonResult)compareTime:(id)aDate
 {
-  return icaltime_compare(_time, ((Date *)aDate)->_time);
+  NSComparisonResult res;
+  int a_isdate = _time.is_date;
+  int b_isdate = ((Date *)aDate)->_time.is_date;
+  _time.is_date = 0;
+  ((Date *)aDate)->_time.is_date = 0;
+  res = icaltime_compare(_time, ((Date *)aDate)->_time);
+  _time.is_date = a_isdate;
+  ((Date *)aDate)->_time.is_date = b_isdate;
+  return res;
 }
 
 - (NSString *)description
@@ -74,45 +117,59 @@
 
 - (id)init
 {
-  const char *tzone;
-  icaltimezone *tz, *utc;
-
   self = [super init];
   if (self) {
-    tzone = [[[[NSCalendarDate calendarDate] timeZone] description] cString];
-    utc = icaltimezone_get_utc_timezone();
-    tz = icaltimezone_get_builtin_timezone(tzone);
-    if (!tz)
-      NSLog(@"Couldn't get a timezone corresponding to %s", tzone);
-    _time = icaltime_current_time_with_zone(NULL);
-    icaltimezone_convert_time(&_time, utc, tz);
-  }
-  return self;
-}
-- (id)initWithCalendarDate:(NSCalendarDate *)cd withTime:(BOOL)time
-{
-  self = [self init];
-  if (self) {
-    [self setDate:!time];
-    _time.year = [cd yearOfCommonEra];
-    _time.month = [cd monthOfYear];
-    _time.day = [cd dayOfMonth];
-    if (time) {
-      _time.hour = [cd hourOfDay];
-      _time.minute = [cd minuteOfHour];
-      _time.second = [cd secondOfMinute];
-    }
+    _time = icaltime_current_time_with_zone(gl_tz);
+    _time = icaltime_set_timezone(&_time, gl_tz);
   }
   return self;
 }
+
 + (id)now
 {
   return AUTORELEASE([[Date alloc] init]);
 }
+
 + (id)today
 {
   Date *d = [[Date alloc] init];
-  [d setDate:YES];
+  [d setIsDate:YES];
+  return AUTORELEASE(d);
+}
+
++ (id)dateWithTimeInterval:(NSTimeInterval)seconds sinceDate:(Date *)refDate
+{
+  Date *d = [[Date alloc] init];
+  
+  d->_time = refDate->_time;
+  /* To be able to add hours and minutes, it has to be a datetime */
+  d-> _time.is_date = 0;
+  d->_time = icaltime_add(d->_time, icaldurationtype_from_int(seconds));
+  /*
+   * FIXME : this is wrong is refDate is a date but seconds
+   * not a multiple of 86400
+   */
+  d->_time.is_date = refDate->_time.is_date;
+  return AUTORELEASE(d);
+}
+
++ (id)dateWithCalendarDate:(NSCalendarDate *)cd withTime:(BOOL)time
+{
+  Date *d = [[Date alloc] init];
+  
+  d->_time.is_date = !time;
+  d->_time.year = [cd yearOfCommonEra];
+  d->_time.month = [cd monthOfYear];
+  d->_time.day = [cd dayOfMonth];
+  if (time) {
+    d->_time.hour = [cd hourOfDay];
+    d->_time.minute = [cd minuteOfHour];
+    d->_time.second = [cd secondOfMinute];
+  } else {
+    d->_time.hour = 0;
+    d->_time.minute = 0;
+    d->_time.second = 0;
+  }
   return AUTORELEASE(d);
 }
 
@@ -124,7 +181,7 @@
 			 hour:_time.hour
 			 minute:_time.minute
 			 second:_time.second
-			 timeZone:nil];
+			 timeZone:gl_nstz];
 }
 
 - (int)year
@@ -155,103 +212,40 @@
 {
   return _time.day;
 }
-/* 0 = sunday */
+/*
+ * 1 : Monday
+ * ...
+ * 7 : Sunday
+ */
 - (int)weekday
 {
-  return icaltime_day_of_week(_time) - 1;
+  int dow = icaltime_day_of_week(_time) - 1;
+  return dow ? dow : 7;
 }
 - (int)weekOfYear
 {
-  /* FIXME : ical function doesn't work. See ticket #3 */
-  return icaltime_week_number(_time);
+  char week[3];
+  time_t tmt = icaltime_as_timet(_time);
+  struct tm *tm = gmtime(&tmt);
+  strftime(week, sizeof(week), "%V", tm);
+  return atoi(week);
 }
 - (int)numberOfDaysInMonth
 {
   return icaltime_days_in_month(_time.month, _time.year);
 }
 
-/* if diff is 23, returns 0 day */
-- (int)daysUntil:(Date *)date
-{
-  struct icaldurationtype dt;
-
-  dt = icaltime_subtract(date->_time, _time);
-  if (dt.is_neg)
-    return -dt.days;
-  return dt.days;
-}
-
-- (int)daysSince:(Date *)date
-{
-  struct icaldurationtype dt;
-
-  dt = icaltime_subtract(_time, date->_time);
-  if (dt.is_neg)
-    return -dt.days;
-  return dt.days;
-}
-
-- (int)weeksUntil:(Date *)date
-{
-  struct icaldurationtype dt;
-
-  dt = icaltime_subtract(date->_time, _time);
-  if (dt.is_neg)
-    return -dt.weeks;
-  return dt.weeks;
-}
-
-- (int)weeksSince:(Date *)date
-{
-  struct icaldurationtype dt;
-
-  dt = icaltime_subtract(_time, date->_time);
-  if (dt.is_neg)
-    return -dt.weeks;
-  return dt.weeks;
-}
-
-- (int)monthsUntil:(Date *)date
-{
-  int months = 0;
-  NSLog(@"monthsUntil");
-  return months;
-}
-
-- (int)monthsSince:(Date *)date
-{
-  int months = 0;
-  NSLog(@"monthsSince");
-  return months;
-}
-
-- (int)yearsUntil:(Date *)date
-{
-  return date->_time.year - _time.year;
-}
-
-- (int)yearSince:(Date *)date
-{
-  return _time.year - date->_time.year;
-}
-
-- (void)clearTime
-{
-  _time.hour = 0;
-  _time.minute = 0;
-  _time.second = 0;
-}
-
 - (void)setSecondOfMinute:(int)second
 {
+  NSAssert(!_time.is_date, @"Works only with datetimes");
   _time.second = second;
   _time = icaltime_normalize(_time);
 }
 
 - (void)setMinute:(int)minute
 {
-  int old = [self minuteOfDay];
-  struct icaldurationtype dt = {0, 0, 0, 0, minute - old, 0};
+  NSAssert(!_time.is_date, @"Works only with datetimes");
+  struct icaldurationtype dt = {0, 0, 0, 0, minute - [self minuteOfDay], 0};
   _time = icaltime_add(_time, dt);
 }
 
@@ -307,31 +301,42 @@
 {
   return _time.is_date;
 }
-- (void)setDate:(BOOL)date
+- (void)setIsDate:(BOOL)date
 {
   _time.is_date = date;
-  if (!date)
-    icaltime_set_timezone(&_time, NULL);
+  if (date) {
+    _time.hour = 0;
+    _time.minute = 0;
+    _time.second = 0;
+  }
+}
+
+- (NSTimeInterval)timeIntervalSince1970
+{
+  return icaltime_as_timet(_time);
+}
+
+- (NSTimeInterval)timeIntervalSinceDate:(Date *)anotherDate
+{
+  return icaldurationtype_as_int(icaltime_subtract(_time, anotherDate->_time));
 }
 @end
 
+/*
+ * Dates are stored in the local timezone in memory
+ * and in utc in iCalTree. These methods do the
+ * necessary conversions
+ */
 @implementation Date(iCalendar)
 - (id)initWithICalTime:(struct icaltimetype)time
 {
   self = [super init];
   if (self)
-    [self setDateToICalTime:time];
+    _time = icaltime_convert_to_zone(time, gl_tz);
   return self;
 }
-
-- (void)setDateToICalTime:(struct icaltimetype)time
-{
-  _time = time;
-}
-
 - (struct icaltimetype)iCalTime
 {
-  return _time;
+  return icaltime_convert_to_zone(_time, gl_utc);
 }
 @end
-
