@@ -4,45 +4,28 @@
 #import "DayView.h"
 #import "ConfigManager.h"
 #import "iCalTree.h"
+#import "AppointmentView.h"
+#import "SelectionManager.h"
+#import "NSColor+SimpleAgenda.h"
 #import "defines.h"
 
-static NSImage *repeatImage;
-
 #define RedimRect(frame) NSMakeRect(frame.origin.x, frame.origin.y, frame.size.width, 10)
 #define TextRect(rect) NSMakeRect(rect.origin.x + 4, rect.origin.y, rect.size.width - 8, rect.size.height - 2)
 
-@interface AppointmentView : NSView
-{
-  Event *_apt;
-  BOOL _selected;
-}
-- (Event *)appointment;
-- (void)setSelected:(BOOL)selected;
+@interface DayView(Private)
+- (NSRect)frameForAppointment:(Event *)apt;
+- (int)minuteToPosition:(int)minutes;
+- (int)positionToMinute:(float)position;
+- (int)roundMinutes:(int)minutes;
+- (int)distanceToDuration:(int)distance;
 @end
 
-@implementation AppointmentView
-- (id)initWithFrame:(NSRect)frameRect appointment:(Event *)apt;
+@interface AppDayView : AppointmentView
 {
-  self = [super initWithFrame:frameRect];
-  if (self) {
-    ASSIGN(_apt, apt);
-    _selected = NO;
-  }
-  return self;
-}
-- (void)dealloc
-{
-  RELEASE(_apt);
-  [super dealloc];
-}
-- (BOOL)selected
-{
-  return _selected;
-}
-- (void)setSelected:(BOOL)selected
-{
-  _selected = selected;
 }
+@end
+
+@implementation AppDayView
 #define CEC_BORDERSIZE 1
 #define RADIUS 5
 - (void)drawRect:(NSRect)rect
@@ -51,15 +34,12 @@ static NSImage *repeatImage;
   NSString *label;
   Date *start = [_apt startDate];
   NSColor *color = [[_apt store] eventColor];
-  NSColor *darkColor = [NSColor colorWithCalibratedRed:[color redComponent] - 0.3
-				green:[color greenComponent] - 0.3
-				blue:[color blueComponent] - 0.3
-				alpha:[color alphaComponent]];
+  NSColor *darkColor = [color colorModifiedWithDeltaRed:-0.3 green:-0.3 blue:-0.3 alpha:-0.3];
   NSDictionary *textAttributes = [NSDictionary dictionaryWithObject:[[_apt store] textColor]
 					       forKey:NSForegroundColorAttributeName];
 
   if ([_apt allDay])
-    title = [NSString stringWithFormat:@"All day : %@", [_apt summary]];
+    title = [NSString stringWithFormat:_(@"All day : %@"), [_apt summary]];
   else
     title = [NSString stringWithFormat:@"%2dh%0.2d : %@", [start hourOfDay], [start minuteOfHour], [_apt summary]];
   if ([_apt text])
@@ -82,7 +62,7 @@ static NSImage *repeatImage;
   PSsetalpha(0.7);
   PSfill();
   PSgrestore();
-  if (_selected)
+  if ([[[SelectionManager globalManager] selection] containsObject:_apt])
     [[NSColor whiteColor] set];
   else
     [darkColor set];
@@ -106,8 +86,8 @@ static NSImage *repeatImage;
     PSfill();
   }
   [label drawInRect:TextRect(rect) withAttributes:textAttributes];
-  if ([_apt interval] != RI_NONE)
-    [repeatImage compositeToPoint:NSMakePoint(rect.size.width - 18, rect.size.height - 18) operation:NSCompositeSourceOver];
+  if ([_apt rrule])
+    [[self repeatImage] compositeToPoint:NSMakePoint(rect.size.width - 18, rect.size.height - 18) operation:NSCompositeSourceOver];
 }
 
 - (void)mouseDown:(NSEvent *)theEvent
@@ -115,16 +95,14 @@ static NSImage *repeatImage;
   DayView *parent = (DayView *)[self superview];
   id delegate = [parent delegate];
   NSPoint mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];
-  int diff;
-  int start;
   int minutes;
   BOOL keepOn = YES;
   BOOL modified = NO;
   BOOL inResize;
 
   if ([theEvent clickCount] > 1) {
-    if ([delegate respondsToSelector:@selector(dayView:editEvent:)])
-      [delegate dayView:parent editEvent:_apt];
+    if ([delegate respondsToSelector:@selector(viewEditEvent:)])
+      [delegate viewEditEvent:_apt];
     return;
   }
   [self becomeFirstResponder];
@@ -134,18 +112,27 @@ static NSImage *repeatImage;
     return;
   inResize = [self mouse:mouseLoc inRect:RedimRect([self bounds])];
   if (inResize) {
+    int startduration = [_apt duration];
+    int starty = [self convertPoint:[theEvent locationInWindow] fromView:self].y;
+    int minStep = [parent minimumStep];
+    int diffduration;
+    int newy;
+
     [[NSCursor resizeUpDownCursor] push];
-    start = [[_apt startDate] minuteOfDay];
     while (keepOn) {
       theEvent = [[self window] nextEventMatchingMask: NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-      mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:self];
+      newy = [self convertPoint:[theEvent locationInWindow] fromView:self].y;
 
       switch ([theEvent type]) {
       case NSLeftMouseDragged:
-	minutes = [parent roundMinutes:[parent positionToMinute:mouseLoc.y] - start];
+	diffduration = [parent distanceToDuration:starty-newy];
+	minutes = [parent roundMinutes:startduration+diffduration];
+	if (minutes < minStep)
+	  minutes = minStep;
 	if (minutes != [_apt duration]) {
 	  [_apt setDuration:minutes];
 	  modified = YES;
+	  [self setFrame:[parent frameForAppointment:_apt]];
 	  [parent setNeedsDisplay:YES];
 	}
 	break;
@@ -157,18 +144,21 @@ static NSImage *repeatImage;
       }
     }
   } else {
+    int startM, currentM, roundedDiffM;
+    Date *date = [[_apt startDate] copy];
+
     [[NSCursor openHandCursor] push];
-    mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:self];
-    diff = [parent minuteToPosition:[[_apt startDate] minuteOfDay]] - mouseLoc.y;
+    startM = [parent positionToMinute:[theEvent locationInWindow].y];
     while (keepOn) {
       theEvent = [[self window] nextEventMatchingMask: NSLeftMouseUpMask | NSLeftMouseDraggedMask];
-      mouseLoc = [self convertPoint:[theEvent locationInWindow] fromView:self];
       switch ([theEvent type]) {
       case NSLeftMouseDragged:
-	minutes = [parent roundMinutes:[parent positionToMinute:mouseLoc.y + diff]];
-	if (minutes != [[_apt startDate] minuteOfDay]) {
-	  [[_apt startDate] setMinute:minutes];
+	currentM = [parent positionToMinute:[theEvent locationInWindow].y];
+	roundedDiffM = [parent roundMinutes:currentM-startM];
+	if (roundedDiffM != 0) {
 	  modified = YES;
+	  [_apt setStartDate:[Date dateWithTimeInterval:roundedDiffM*60 sinceDate:date]];
+	  [self setFrame:[parent frameForAppointment:_apt]];
 	  [parent setNeedsDisplay:YES];
 	}
 	break;
@@ -179,18 +169,17 @@ static NSImage *repeatImage;
 	break;
       }
     }
+    [date release];
   }
   [NSCursor pop];
-  if (modified && [delegate respondsToSelector:@selector(dayView:modifyEvent:)])
-    [delegate dayView:parent modifyEvent:_apt];
+  if (modified && [delegate respondsToSelector:@selector(viewModifyEvent:)])
+    [delegate viewModifyEvent:_apt];
 }
-- (Event *)appointment
+
+- (void)resizeWithOldSuperviewSize:(NSSize)oldBoundsSize
 {
-  return _apt;
-}
-- (BOOL)acceptsFirstResponder
-{
-  return YES;
+  DayView *parent = (DayView *)[self superview];
+  [self setFrame:[parent frameForAppointment:_apt]];
 }
 @end
 
@@ -202,10 +191,8 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
 @implementation DayView
 - (NSDictionary *)defaults
 {
-  NSDictionary *dict = [NSDictionary 
-			 dictionaryWithObjects:[NSArray arrayWithObjects:@"9", @"18", @"15", nil]
-			 forKeys:[NSArray arrayWithObjects:FIRST_HOUR, LAST_HOUR, MIN_STEP, nil]];
-  return dict;
+  return [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"9", @"18", @"15", nil]
+		       forKeys:[NSArray arrayWithObjects:FIRST_HOUR, LAST_HOUR, MIN_STEP, nil]];
 }
 
 - (id)initWithFrame:(NSRect)frameRect
@@ -221,52 +208,46 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
     _firstH = [config integerForKey:FIRST_HOUR];
     _lastH = [config integerForKey:LAST_HOUR];
     _minStep = [config integerForKey:MIN_STEP];
-    _textAttributes = [[NSDictionary dictionaryWithObject:[NSColor textColor]
- 				     forKey:NSForegroundColorAttributeName] retain];
+    _textAttributes = [[NSDictionary dictionaryWithObject:[NSColor textColor] forKey:NSForegroundColorAttributeName] retain];
     _backgroundColor = [[[NSColor controlBackgroundColor] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] retain];
-    _alternateBackgroundColor = [[NSColor colorWithCalibratedRed:[_backgroundColor redComponent] + 0.05
-					 green:[_backgroundColor greenComponent] + 0.05
-					 blue:[_backgroundColor blueComponent] + 0.05
-					 alpha:[_backgroundColor alphaComponent]] retain];
-
-    if (!repeatImage) {
-      NSString *path = [[NSBundle mainBundle] pathForImageResource:@"repeat"];
-      repeatImage = [[NSImage alloc] initWithContentsOfFile:path];
-      [repeatImage setFlipped:YES];
-    }
-    [self reloadData];
+    _alternateBackgroundColor = [[_backgroundColor colorModifiedWithDeltaRed:0.05 green:0.05 blue:0.05 alpha:0] retain];
   }
   return self;
 }
 
 - (void)dealloc
 {
+  [[ConfigManager globalConfig] unregisterClient:self];
   [_backgroundColor release];
   [_alternateBackgroundColor release];
   [_textAttributes release];
+  RELEASE(_date);
   [super dealloc];
 }
 
-/* FIXME : the following could probably be simplified... */
-- (int)_minuteToSize:(int)minutes
+- (int)minuteToSize:(int)minutes
 {
   return minutes * [self frame].size.height / ((_lastH - _firstH + 1) * 60);
 }
 - (int)minuteToPosition:(int)minutes
 {
-  return [self frame].size.height - [self _minuteToSize:minutes - (_firstH * 60)] - 1;
+  return [self frame].size.height - [self minuteToSize:minutes - (_firstH * 60)] - 1;
 }
 - (int)positionToMinute:(float)position
 {
   return ((_lastH + 1) * 60) - ((_lastH - _firstH + 1) * 60) * position / [self frame].size.height;
 }
+- (int)distanceToDuration:(int)distance
+{
+  return ((_lastH - _firstH + 1) * 60) * distance / [self frame].size.height;
+}
 
 - (void)fixFrames
 {
   int i, j, k, n;
   NSArray *subviews;
   AppointmentView *view, *next;
-  NSRect fview, fnext;
+  NSRect fview, fnext, frame;
   float width, x;
 
   subviews = [[self subviews] sortedArrayUsingFunction:compareAppointmentViews context:NULL];
@@ -284,8 +265,9 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
       fview = NSUnionRect(fview, fnext);
     }
     if (n != 1) {
-      width = [view frame].size.width / n;
-      x = [view frame].origin.x;
+      frame = [self frameForAppointment:[view appointment]];
+      width  = frame.size.width / n;
+      x = frame.origin.x;
       for (k = i; k < i + n; k++) {
 	view = [subviews objectAtIndex:k];
 	fview = [view frame];
@@ -297,45 +279,39 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
     }
   }
 }
+
 - (NSRect)frameForAppointment:(Event *)apt
 {
+  NSRange range;
   int size, start;
 
   if ([apt allDay])
     return NSMakeRect(40, 0, [self frame].size.width - 48, [self frame].size.height);
-  start = [self minuteToPosition:[[apt startDate] minuteOfDay]];
-  size = [self _minuteToSize:[apt duration]];
+  range = [apt intersectionWithDay:_date];
+  start = [self minuteToPosition:range.location/60];
+  size = [self minuteToSize:range.length/60];
   return NSMakeRect(40, start - size, [self frame].size.width - 48, size);
 }
+
 - (int)roundMinutes:(int)minutes
 {
-  int rounded = minutes / _minStep * _minStep;
-  return (rounded < _minStep) ? _minStep : rounded;
+  return minutes / _minStep * _minStep;
 }
 
 - (void)drawRect:(NSRect)rect
 {
   NSSize size;
   NSString *hour;
-  AppointmentView *aptv; 
-  NSEnumerator *enumerator;
   int h, start;
   int hrow;
   float miny, maxy;
 
-  /* 
-   * FIXME : this is ugly and slow, we're doing
-   * work when it's not needed and probably twice.
-   */
-  enumerator = [[self subviews] objectEnumerator];
-  while ((aptv = [enumerator nextObject]))
-    [aptv setFrame:[self frameForAppointment:[aptv appointment]]];
   [self fixFrames];
   /*
    * FIXME : if we draw the string in the same
    * loop it doesn't appear on the screen.
    */
-  hrow = [self _minuteToSize:60];
+  hrow = [self minuteToSize:60];
   for (h = _firstH; h <= _lastH + 1; h++) {
     start = [self minuteToPosition:h * 60];
     if (h % 2)
@@ -358,14 +334,6 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   }
 }
 
-- (id)dataSource
-{
-  return dataSource;
-}
-- (void)setDataSource:(id)source
-{
-  dataSource = source;
-}
 - (id)delegate
 {
   return delegate;
@@ -377,33 +345,28 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
 
 - (void)selectAppointmentView:(AppointmentView *)aptv
 {
-  if (_selected != aptv) {
-    [_selected setSelected:NO];
-    [aptv setSelected:YES];
-    [self setNeedsDisplay:YES];
-    _selected = aptv;
-    if ([delegate respondsToSelector:@selector(dayView:selectEvent:)])
-      [delegate dayView:self selectEvent:[aptv appointment]];
-  }
+  _selected = aptv;
+  [[SelectionManager globalManager] set:[aptv appointment]];
+  if ([delegate respondsToSelector:@selector(viewSelectEvent:)])
+    [delegate viewSelectEvent:[aptv appointment]];
+  [self setNeedsDisplay:YES];
 }
 
 - (void)reloadData
 {
-  ConfigManager *config = [ConfigManager globalConfig];
   NSEnumerator *enumerator, *enm;
   AppointmentView *aptv;
   Event *apt;
   NSSet *events;
   BOOL found;
 
-  events = [dataSource scheduledAppointmentsForDay:nil];
+  events = [[StoreManager globalManager] scheduledAppointmentsForDay:_date];
   enumerator = [[self subviews] objectEnumerator];
   while ((aptv = [enumerator nextObject])) {
     if (![events containsObject:[aptv appointment]] || ![[[aptv appointment] store] displayed]) {
       if (aptv == _selected)
 	_selected = nil;
       [aptv removeFromSuperviewWithoutNeedingDisplay];
-      [aptv release];
     }
   }
   enumerator = [events objectEnumerator];
@@ -411,16 +374,15 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
     found = NO;
     enm = [[self subviews] objectEnumerator];
     while ((aptv = [enm nextObject])) {
+      [aptv setFrame:[self frameForAppointment:[aptv appointment]]];
       if ([apt isEqual:[aptv appointment]]) {
 	found = YES;
 	break;
       }
     }
     if (found == NO) {
-      /* FIXME : probably shouldn't be there */
-      [config registerClient:self forKey:[[apt store] description]];
       if ([[apt store] displayed])
-	[self addSubview:[[AppointmentView alloc] initWithFrame:NSZeroRect appointment:apt]];
+	[self addSubview:AUTORELEASE([[AppDayView alloc] initWithFrame:[self frameForAppointment:apt]  appointment:apt])];
     }
   }
   [self setNeedsDisplay:YES];
@@ -452,8 +414,8 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   if (ABS(_startPt.y - _endPt.y) > 7 && [self mouse:_endPt inRect:[self bounds]]) {
     start = [self positionToMinute:MAX(_startPt.y, _endPt.y)];
     end = [self positionToMinute:MIN(_startPt.y, _endPt.y)];
-    if ([delegate respondsToSelector:@selector(dayView:createEventFrom:to:)])
-      [delegate dayView:self createEventFrom:[self roundMinutes:start] to:[self roundMinutes:end]];
+    if ([delegate respondsToSelector:@selector(viewCreateEventFrom:to:)])
+      [delegate viewCreateEventFrom:[self roundMinutes:start] to:[self roundMinutes:end]];
   }
   _startPt = _endPt = NSMakePoint(0, 0);
   [self setNeedsDisplay:YES];
@@ -470,18 +432,17 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   switch (character) {
   case '\r':
   case NSEnterCharacter: 
-    NSLog(@"enter");
     if (_selected != nil) {
-      if ([delegate respondsToSelector:@selector(dayView:editEvent:)])
-	[delegate dayView:self editEvent:[_selected appointment]];
-      return;
+      if ([delegate respondsToSelector:@selector(viewEditEvent:)])
+	[delegate viewEditEvent:[_selected appointment]];
     }
+    break;
   case NSUpArrowFunctionKey:
     [self moveUp:self];
-    return;
+    break;
   case NSDownArrowFunctionKey:
     [self moveDown:self];
-    return;
+    break;
   case NSTabCharacter:
     if (_selected != nil) {
       unsigned int index = [[self subviews] indexOfObject:_selected];
@@ -496,11 +457,14 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
 	    index = 0;
 	}
 	[self selectAppointmentView:[[self subviews] objectAtIndex:index]];
-      }
-      return;
+      } 
+    } else {
+      [self selectAppointmentView:[[self subviews] lastObject]];
     }
+    break;
+  default:
+    [super keyDown:theEvent];
   }
-  [super keyDown:theEvent];
 }
 
 - (void)moveUp:(id)sender
@@ -508,8 +472,8 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   if (_selected != nil) {
     [[[_selected appointment] startDate] changeMinuteBy:-_minStep];
     [_selected setFrame:[self frameForAppointment:[_selected appointment]]];
-    if ([delegate respondsToSelector:@selector(dayView:modifyEvent:)])
-      [delegate dayView:self modifyEvent:[_selected appointment]];
+    if ([delegate respondsToSelector:@selector(viewModifyEvent:)])
+      [delegate viewModifyEvent:[_selected appointment]];
   }
 }
 
@@ -518,8 +482,8 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   if (_selected != nil) {
     [[[_selected appointment] startDate] changeMinuteBy:_minStep];
     [_selected setFrame:[self frameForAppointment:[_selected appointment]]];
-    if ([delegate respondsToSelector:@selector(dayView:modifyEvent:)])
-      [delegate dayView:self modifyEvent:[_selected appointment]];
+    if ([delegate respondsToSelector:@selector(viewModifyEvent:)])
+      [delegate viewModifyEvent:[_selected appointment]];
   }
 }
 
@@ -528,12 +492,15 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   return YES;
 }
 
-- (void)config:(ConfigManager*)config dataDidChangedForKey:(NSString *)key
+- (void)config:(ConfigManager *)config dataDidChangedForKey:(NSString *)key
 {
-  _firstH = [config integerForKey:FIRST_HOUR];
-  _lastH = [config integerForKey:LAST_HOUR];
-  _minStep = [config integerForKey:MIN_STEP];
-  [self reloadData];
+  if ([key isEqualToString:MIN_STEP])
+    _minStep = [config integerForKey:MIN_STEP];
+  else {
+    _firstH = [config integerForKey:FIRST_HOUR];
+    _lastH = [config integerForKey:LAST_HOUR];
+    [self reloadData];
+  }
 }
 
 - (int)firstHour
@@ -549,12 +516,9 @@ NSComparisonResult compareAppointmentViews(id a, id b, void *data)
   return _minStep;
 }
 
-- (void)deselectAll:(id)sender
+- (void)setDate:(Date *)date
 {
-  if (_selected) {
-    [_selected setSelected:NO];
-    [self setNeedsDisplay:YES];
-    _selected = nil;
-  }
+  ASSIGNCOPY(_date, date);
+  [self reloadData];
 }
 @end
