@@ -1,16 +1,20 @@
 /* emacs buffer mode hint -*- objc -*- */
 
-#import <ical.h>
-#import "AgendaStore.h"
+#import "config.h"
+#import "MemoryStore.h"
 #import "Date.h"
 
-enum classificationType
-{
-  CT_NONE = 0, 
-  CT_PUBLIC, 
-  CT_PRIVATE, 
-  CT_CONFIDENTIAL 
-};
+/*
+  icalproperty_class
+
+  ICAL_CLASS_X = 10006,
+  ICAL_CLASS_PUBLIC = 10007,
+  ICAL_CLASS_PRIVATE = 10008,
+  ICAL_CLASS_CONFIDENTIAL = 10009,
+  ICAL_CLASS_NONE = 10010
+*/
+
+@class SAAlarm;
 
 @interface Element : NSObject <NSCoding>
 {
@@ -18,8 +22,10 @@ enum classificationType
   NSString *_uid;
   NSString *_summary;
   NSAttributedString *_text;
-  enum classificationType _classification;
+  icalproperty_class _classification;
   Date *_stamp;
+  NSMutableArray *_alarms;
+  NSMutableArray *_categories;
 }
 
 - (id)initWithSummary:(NSString *)summary;
@@ -28,16 +34,27 @@ enum classificationType
 - (NSAttributedString *)text;
 - (NSString *)summary;
 - (NSString *)UID;
-- (enum classificationType)classification;
+- (icalproperty_class)classification;
 - (Date *)dateStamp;
 
 - (void)setStore:(id <MemoryStore>)store;
 - (void)setText:(NSAttributedString *)text;
 - (void)setSummary:(NSString *)summary;
 - (void)setUID:(NSString *)uid;
-- (void)setClassification:(enum classificationType)classification;
+- (void)setClassification:(icalproperty_class)classification;
 - (void)setDateStamp:(Date *)stamp;
 
+- (BOOL)hasAlarms;
+- (NSArray *)alarms;
+- (void)addAlarm:(SAAlarm *)alarm;
+- (void)removeAlarm:(SAAlarm *)alarm;
+
+- (NSArray *)categories;
+- (void)setCategories:(NSArray *)categories;
+- (void)addCategory:(NSString *)category;
+- (void)removeCategory:(NSString *)category;
+- (BOOL)inCategory:(NSString *)category;
+
 - (id)initWithICalComponent:(icalcomponent *)ic;
 - (icalcomponent *)asICalComponent;
 - (void)deleteProperty:(icalproperty_kind)kind fromComponent:(icalcomponent *)ic;
