@@ -1,35 +1,29 @@
-/*
- * Based on ChronographerSource Appointment class
- */
-
 #import <Foundation/Foundation.h>
 #import "Event.h"
+#import "DateRange.h"
 
 @implementation Event(NSCoding)
--(void)encodeWithCoder:(NSCoder *)coder
+- (void)encodeWithCoder:(NSCoder *)coder
 {
   [super encodeWithCoder:coder];
-  [coder encodeObject:startDate forKey:@"sdate"];
-  [coder encodeObject:endDate forKey:@"edate"];
-  [coder encodeInt:interval forKey:@"interval"];
-  [coder encodeInt:frequency forKey:@"frequency"];
-  [coder encodeInt:duration forKey:@"duration"];
+  [coder encodeObject:_startDate forKey:@"sdate"];
+  [coder encodeInt:_duration forKey:@"duration"];
   [coder encodeObject:_location forKey:@"location"];
   [coder encodeBool:_allDay forKey:@"allDay"];
+  [coder encodeObject:_rrule forKey:@"rrule"];
 }
--(id)initWithCoder:(NSCoder *)coder
+- (id)initWithCoder:(NSCoder *)coder
 {
   [super initWithCoder:coder];
-  startDate = [[coder decodeObjectForKey:@"sdate"] retain];
-  endDate = [[coder decodeObjectForKey:@"edate"] retain];
-  interval = [coder decodeIntForKey:@"interval"];
-  frequency = [coder decodeIntForKey:@"frequency"];
-  duration = [coder decodeIntForKey:@"duration"];
+  _startDate = [[coder decodeObjectForKey:@"sdate"] retain];
+  _duration = [coder decodeIntForKey:@"duration"];
   _location = [[coder decodeObjectForKey:@"location"] retain];
   if ([coder containsValueForKey:@"allDay"])
     _allDay = [coder decodeBoolForKey:@"allDay"];
   else
     _allDay = NO;
+  if ([coder containsValueForKey:@"rrule"])
+    _rrule = [[coder decodeObjectForKey:@"rrule"] retain];
   return self;
 }
 @end
@@ -37,15 +31,14 @@
 @implementation Event
 - (id)copy
 {
-  NSData *data = [NSKeyedArchiver archivedDataWithRootObject:self];
-  Event *new = [NSKeyedUnarchiver unarchiveObjectWithData:data];
+  Event *new = [NSKeyedUnarchiver unarchiveObjectWithData:[NSKeyedArchiver archivedDataWithRootObject:self]];
   [new generateUID];
   return new;
 }
 
 - (id)initWithStartDate:(Date *)start duration:(int)minutes title:(NSString *)aTitle
 {
-  self = [self initWithSummary:aTitle];
+  self = [super initWithSummary:aTitle];
   if (self) {
     [self setStartDate:start];
     [self setDuration:minutes];
@@ -55,57 +48,37 @@
 
 - (void)dealloc
 {
-  [super dealloc];
   RELEASE(_location);
-  RELEASE(startDate);
-  RELEASE(endDate);
+  RELEASE(_startDate);
+  RELEASE(_rrule);
+  [super dealloc];
+}
+
+- (BOOL)isScheduledForDay:(Date *)day
+{
+  return [self intersectionWithDay:day].length > 0;
 }
 
 /*
- * Code adapted from ChronographerSource Appointment:isScheduledFor
+ * Values are in seconds whereas time granularity in
+ * all other Event methods is minute.
+ * Worth changing ?
  */
-- (BOOL)isScheduledForDay:(Date *)day
+- (NSRange)intersectionWithDay:(Date *)day
 {
-  NSAssert(day != nil, @"Empty day argument");
-  if ([day daysUntil:startDate] > 0 || (endDate && [day daysSince:endDate] > 0))
-    return NO;
-  switch (interval) {
-  case RI_NONE:
-    return [day compare:startDate] == 0;
-  case RI_DAILY:
-    return ((frequency == 1) ||
-	    ([startDate daysUntil: day] % frequency) == 0);
-  case RI_WEEKLY:
-    return (([startDate weekday] == [day weekday]) &&
-	    ((frequency == 1) ||
-	     (([startDate weeksUntil: day] % frequency) == 0)));
-  case RI_MONTHLY:
-    return (([startDate dayOfMonth] == [day dayOfMonth]) &&
-	    ((frequency == 1) ||
-	     (([startDate monthsUntil: day] % frequency) == 0)));
-  case RI_YEARLY:
-    return ((([startDate dayOfMonth] == [day dayOfMonth]) &&
-	     ([startDate monthOfYear] == [day monthOfYear])) &&
-	    ((frequency == 1) ||
-	     (([startDate yearsUntil: day] % frequency) == 0)));
-  }
-  return NO;
-}
+  NSEnumerator *enumerator;
+  DateRange *range = AUTORELEASE([[DateRange alloc] initWithStart:_startDate duration:[self duration]*60]);
 
-- (BOOL)isScheduledBetweenDay:(Date *)start andDay:(Date *)end
-{
-  int nd;
-  Date *work = [start copy];
-
-  for (nd = 0; nd < [start daysUntil:end] + 1; nd++) {
-    if ([self isScheduledForDay:work]) {
-      [work release];
-      return YES;
-    }
-    [work incrementDay];
+  if (!_rrule)
+    return [range intersectionWithDay:day];
+  enumerator = [_rrule enumeratorFromDate:_startDate length:[self duration]*60];
+  while ((range = [enumerator nextObject])) {
+    if ([range intersectsWithDay:day])
+      return [range intersectionWithDay:day];
+    if ([[range start] compare:day] > 0)
+      break;
   }
-  [work release];
-  return NO;
+  return NSMakeRange(0, 0);
 }
 
 - (NSString *)location
@@ -122,22 +95,28 @@
 {
   return _allDay;
 }
+
 - (void)setAllDay:(BOOL)allDay
 {
-  [startDate setDate:allDay];
+  /*
+   * FIXME : why do we force startDate to being a date ?
+   * What is the relation with the appointment being an
+   * all day one ?
+   */
+  [_startDate setIsDate:allDay];
   _allDay = allDay;
 }
 
 - (NSString *)description
 {
-  return [NSString stringWithFormat:@"<%@> from <%@> for <%d> to <%@> (%d)", [self summary], [startDate description], [self duration], [endDate description], interval];
+  return [NSString stringWithFormat:@"<%@> from <%@> for <%d>", [self summary], [_startDate description], [self duration]];
 }
 
 - (NSString *)details
 {
-  if ([self allDay])
+  if (_allDay)
     return @"all day";
-  int minute = [[self startDate] minuteOfDay];
+  int minute = [_startDate minuteOfDay];
   return [NSString stringWithFormat:@"%dh%02d", minute / 60, minute % 60];
 }
 
@@ -154,52 +133,44 @@
 
 - (int)duration
 {
-  return duration;
-}
-
-- (int)frequency
-{
-  return frequency;
+  if (_allDay)
+    return 1440;
+  return _duration;
 }
 
 - (Date *)startDate
 {
-  return startDate;
+  return _startDate;
 }
 
-- (Date *)endDate
+- (RecurrenceRule *)rrule
 {
-  return endDate;
-}
-
-- (int)interval
-{
-  return interval;
+  return _rrule;
 }
 
 - (void)setDuration:(int)newDuration
 {
-  duration = newDuration;
-}
-
-- (void)setFrequency:(int)newFrequency
-{
-  frequency = newFrequency;
+  _duration = newDuration;
 }
 
 - (void)setStartDate:(Date *)newStartDate
 {
-  ASSIGNCOPY(startDate, newStartDate);
+  ASSIGNCOPY(_startDate, newStartDate);
 }
 
-- (void)setEndDate:(Date *)date
+- (void)setRRule:(RecurrenceRule *)arule
 {
-  ASSIGNCOPY(endDate, date);
+  ASSIGN(_rrule, arule);
 }
 
-- (void)setInterval:(int)newInterval
+- (NSEnumerator *)dateEnumerator
 {
-  interval = newInterval;
+  return [_rrule enumeratorFromDate:_startDate];
+}
+
+- (NSEnumerator *)dateRangeEnumerator
+{
+  return [_rrule enumeratorFromDate:_startDate length:_allDay?86400:_duration*60];
 }
 @end
 
@@ -213,7 +184,6 @@
   struct icaltimetype start;
   struct icaltimetype end;
   struct icaldurationtype diff;
-  struct icalrecurrencetype rec;
   Date *date;
   const char *location;
 
@@ -242,40 +212,13 @@
     }
     [self setDuration:icaldurationtype_as_int(diff) / 60];
   }
-
   prop = icalcomponent_get_first_property(ic, ICAL_RRULE_PROPERTY);
-  if (prop) {
-    rec = icalproperty_get_rrule(prop);
-    switch (rec.freq) {
-    case ICAL_DAILY_RECURRENCE:
-      [self setInterval:RI_DAILY];
-      [self setFrequency:rec.interval];
-      break;
-    case ICAL_WEEKLY_RECURRENCE:
-      [self setInterval:RI_WEEKLY];
-      [self setFrequency:rec.interval];
-      break;
-    case ICAL_MONTHLY_RECURRENCE:
-      [self setInterval:RI_MONTHLY];
-      [self setFrequency:rec.interval];
-      break;
-    case ICAL_YEARLY_RECURRENCE:
-      [self setInterval:RI_YEARLY];
-      [self setFrequency:rec.interval];
-      break;
-    default:
-      NSLog(@"ToDo");
-      break;
-    }
-    if (!icaltime_is_null_time(rec.until)) {
-      [date setDateToICalTime:rec.until];
-      [self setEndDate:date];
-    }
-  }
+  if (prop)
+    _rrule = [[RecurrenceRule alloc] initWithICalRRule:icalproperty_get_rrule(prop)];
   [date release];
   location = icalcomponent_get_location(ic);
   if (location)
-    [self setLocation:[NSString stringWithCString:location encoding:NSUTF8StringEncoding]];
+    [self setLocation:[NSString stringWithUTF8String:location]];
   return self;
 
  init_error:
@@ -301,7 +244,6 @@
 - (BOOL)updateICalComponent:(icalcomponent *)ic
 {
   Date *end;
-  struct icalrecurrencetype irec;
   icalproperty *prop;
 
   if (![super updateICalComponent:ic])
@@ -312,14 +254,14 @@
     icalcomponent_add_property(ic, icalproperty_new_location([[self location] UTF8String]));
 
   [self deleteProperty:ICAL_DTSTART_PROPERTY fromComponent:ic];
-  icalcomponent_add_property(ic, icalproperty_new_dtstart([startDate iCalTime]));
+  icalcomponent_add_property(ic, icalproperty_new_dtstart([_startDate iCalTime]));
 
   [self deleteProperty:ICAL_DTEND_PROPERTY fromComponent:ic];
   [self deleteProperty:ICAL_DURATION_PROPERTY fromComponent:ic];
   if (![self allDay])
-    icalcomponent_add_property(ic, icalproperty_new_dtend(icaltime_add([startDate iCalTime], icaldurationtype_from_int(duration * 60))));
+    icalcomponent_add_property(ic, icalproperty_new_dtend(icaltime_add([_startDate iCalTime], icaldurationtype_from_int(_duration * 60))));
   else {
-    end = [startDate copy];
+    end = [_startDate copy];
     [end incrementDay];
     icalcomponent_add_property(ic, icalproperty_new_dtend([end iCalTime]));
     [end release];
@@ -335,30 +277,8 @@
   }
 
   [self deleteProperty:ICAL_RRULE_PROPERTY fromComponent:ic];
-  if (interval != RI_NONE) {
-    icalrecurrencetype_clear(&irec);
-    switch (interval) {
-    case RI_DAILY:
-      irec.freq = ICAL_DAILY_RECURRENCE;
-      break;
-    case RI_WEEKLY:
-      irec.freq = ICAL_WEEKLY_RECURRENCE;
-      break;
-    case RI_MONTHLY:
-      irec.freq = ICAL_MONTHLY_RECURRENCE;
-      break;
-    case RI_YEARLY:
-      irec.freq = ICAL_YEARLY_RECURRENCE;
-      break;
-    default:
-      NSLog(@"ToDo");
-    }
-    if (endDate != nil)
-      irec.until = [endDate iCalTime];
-    else
-      irec.until = icaltime_null_time();
-    icalcomponent_add_property(ic, icalproperty_new_rrule(irec));
-  }
+  if (_rrule)
+    icalcomponent_add_property(ic, icalproperty_new_rrule([_rrule iCalRRule]));
   return YES;
 }
 
@@ -367,4 +287,3 @@
   return ICAL_VEVENT_COMPONENT;
 }
 @end
-
