@@ -1,40 +1,46 @@
 #import <AppKit/AppKit.h>
-#import "LocalStore.h"
+#import "AgendaStore.h"
 #import "Event.h"
 #import "Task.h"
 #import "defines.h"
 
-#define CurrentVersion 2
-#define LocalAgendaPath @"~/GNUstep/Library/SimpleAgenda"
+@interface LocalStore : MemoryStore <AgendaStore>
+{
+  NSString *_globalPath;
+  NSString *_globalFile;
+  NSString *_globalTaskFile;
+}
+@end
 
 @implementation LocalStore
-
 - (NSDictionary *)defaults
 {
-  return [NSDictionary dictionaryWithObjectsAndKeys:
-			 [NSArchiver archivedDataWithRootObject:[NSColor yellowColor]], ST_COLOR,
-			 [NSArchiver archivedDataWithRootObject:[NSColor darkGrayColor]], ST_TEXT_COLOR,
+  return [NSDictionary dictionaryWithObjectsAndKeys:[[NSColor yellowColor] description], ST_COLOR,
+		       [[NSColor darkGrayColor] description], ST_TEXT_COLOR,
 		       [NSNumber numberWithBool:YES], ST_RW,
 		       [NSNumber numberWithBool:YES], ST_DISPLAY,
-		       [NSNumber numberWithInt:CurrentVersion], ST_VERSION,
+		       [NSNumber numberWithBool:YES], ST_ENABLED,
 		       nil, nil];
 }
 
 - (id)initWithName:(NSString *)name
 {
-  NSString *filename;
-
   self = [super initWithName:name];
   if (self) {
-    filename = [_config objectForKey:ST_FILE];
-    _globalPath = [[LocalAgendaPath stringByExpandingTildeInPath] retain];
-    _globalFile = [[NSString pathWithComponents:[NSArray arrayWithObjects:_globalPath, filename, nil]] retain];
+    _globalPath = [[[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] 
+		     stringByAppendingPathComponent:@"SimpleAgenda"] retain];
+    _globalFile = [[_globalPath stringByAppendingPathComponent:[_config objectForKey:ST_FILE]] retain];
     _globalTaskFile = [[NSString stringWithFormat:@"%@.tasks", _globalFile] retain];
     [self read];
   }
   return self;
 }
 
++ (BOOL)isUserInstanciable
+{
+  return YES;
+}
+
 + (BOOL)registerWithName:(NSString *)name
 {
   ConfigManager *cm;
@@ -42,6 +48,7 @@
   cm = [[ConfigManager alloc] initForKey:name withParent:nil];
   [cm setObject:[name copy] forKey:ST_FILE];
   [cm setObject:[[self class] description] forKey:ST_CLASS];
+  [cm release];
   return YES;
 }
 
@@ -59,17 +66,16 @@
   [super dealloc];
 }
 
-- (BOOL)read
+- (void)read
 {
   NSFileManager *fm = [NSFileManager defaultManager];
   NSSet *savedData;
   BOOL isDir;
-  int version;
 
   if (![fm fileExistsAtPath:_globalPath]) {
     if (![fm createDirectoryAtPath:_globalPath attributes:nil]) {
       NSLog(@"Error creating dir %@", _globalPath);
-      return NO;
+      return;
     }
     NSLog(@"Created directory %@", _globalPath);
   }
@@ -78,11 +84,6 @@
     if (savedData) {
       [self fillWithElements:savedData];
       NSLog(@"LocalStore from %@ : loaded %d appointment(s)", _globalFile, [[self events] count]);
-      version = [_config integerForKey:ST_VERSION];
-      if (version < CurrentVersion) {
-	[_config setInteger:CurrentVersion forKey:ST_VERSION];
-	[self write];
-      }
     }
   }
   if ([fm fileExistsAtPath:_globalTaskFile isDirectory:&isDir] && !isDir) {
@@ -92,7 +93,6 @@
       NSLog(@"LocalStore from %@ : loaded %d tasks(s)", _globalTaskFile, [[self tasks] count]);
     }
   }
-  return YES;
 }
 
 - (BOOL)write
