@@ -6,55 +6,72 @@
 #import "defines.h"
 
 @implementation PreferencesController
-
--(id)initWithStoreManager:(StoreManager *)sm
+- (id)init
 {
   self = [super init];
   if (self) {
     if (![NSBundle loadNibNamed:@"Preferences" owner:self])
       return nil;
 
-    ASSIGN(_sm, sm);
+    _sm = [StoreManager globalManager];
     HourFormatter *formatter = [[[HourFormatter alloc] init] autorelease];
     [[dayStartText cell] setFormatter:formatter];
     [[dayEndText cell] setFormatter:formatter];
     [[minStepText cell] setFormatter:formatter];
+    [[refreshIntervalText cell] setFormatter:formatter];
     RETAIN(globalPreferences);
     RETAIN(storePreferences);
     RETAIN(storeFactory);
     [self selectItem:itemPopUp];
     [panel setFrameAutosaveName:@"preferencesPanel"];
-    /* FIXME : I can't set these within Gorm */
-    [dayStart setContinuous:YES];
-    [dayEnd setContinuous:YES];
-    [minStep setContinuous:YES];
+    /* FIXME : could we call setupDefaultStore directly ? */
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					  selector:@selector(storeStateChanged:) 
+					  name:SAStatusChangedForStore 
+					  object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+ 					  selector:@selector(storeStateChanged:) 
+ 					  name:SAEnabledStatusChangedForStore 
+					  object:nil];
   }
   return self;
 }
 
 - (void)dealloc
 {
-  RELEASE(_sm);
   RELEASE(globalPreferences);
   RELEASE(storePreferences);
   RELEASE(storeFactory);
   [super dealloc];
 }
 
--(void)_setupStores
+- (void)setupDefaultStore
 {
-  ConfigManager *config = [ConfigManager globalConfig];
-  NSString *defaultStore = [config objectForKey:ST_DEFAULT];
+  NSString *defaultStore = [[ConfigManager globalConfig] objectForKey:ST_DEFAULT];
   NSEnumerator *list = [_sm storeEnumerator];
   id <AgendaStore> aStore;
 
   [defaultStorePopUp removeAllItems];
   while ((aStore = [list nextObject])) {
-    if ([aStore writable])
+    if ([aStore writable] && [aStore enabled])
       [defaultStorePopUp addItemWithTitle:[aStore description]];
   }
-  [defaultStorePopUp selectItemWithTitle:defaultStore];
+  if ([defaultStorePopUp numberOfItems] > 0) {
+    if ([defaultStorePopUp itemWithTitle:defaultStore])
+      [defaultStorePopUp selectItemWithTitle:defaultStore];
+    else {
+      [defaultStorePopUp selectItemAtIndex:0];
+      [self selectDefaultStore:self];
+    }
+  }
+}
 
+- (void)setupStores
+{
+  NSEnumerator *list;
+  id <AgendaStore> aStore;
+
+  [self setupDefaultStore];
   list = [_sm storeEnumerator];
   [storePopUp removeAllItems];
   while ((aStore = [list nextObject]))
@@ -63,7 +80,12 @@
   [self selectStore:self];
 }
 
--(void)showPreferences
+- (void)storeStateChanged:(NSNotification *)notification
+{
+  [self setupDefaultStore];
+}
+
+- (void)showPreferences
 {
   NSEnumerator *backends = [[StoreManager backends] objectEnumerator];
   ConfigManager *config = [ConfigManager globalConfig];
@@ -78,45 +100,70 @@
   [dayEndText setIntValue:end];
   [minStep setDoubleValue:step/60.0];
   [minStepText setDoubleValue:step/60.0];
+  [showTooltip setState:[config integerForKey:TOOLTIP]];
+  [showDateAppIcon setState:[config integerForKey:APPICON_DATE]];
+  [showTimeAppIcon setState:[config integerForKey:APPICON_TIME]];
 
-  [self _setupStores];
+  [self setupStores];
   [storeClass removeAllItems];
   while ((backend = [backends nextObject]))
-    [storeClass addItemWithTitle:[backend storeTypeName]];
+    if ([backend isUserInstanciable])
+      [storeClass addItemWithTitle:[backend storeTypeName]];
   [storeClass selectItemAtIndex:0];
   [createButton setEnabled:NO];
   [panel makeKeyAndOrderFront:self];
 }
 
 
--(void)selectStore:(id)sender
+- (void)periodicSetupForStore:(id)store
+{
+  if ([store conformsToProtocol:@protocol(PeriodicRefresh)]) {
+    [storeRefresh setEnabled:YES];
+    [storeRefresh setState:[store periodicRefresh]];
+    [refreshInterval setEnabled:[store periodicRefresh]];
+    [refreshIntervalText setEnabled:[store periodicRefresh]];
+    [refreshIntervalText setDoubleValue:[store refreshInterval]/3600.0];
+    [refreshInterval setDoubleValue:[store refreshInterval]/3600.0];
+  } else {
+    [storeRefresh setEnabled:NO];
+    [storeRefresh setState:NO];
+    [refreshInterval setEnabled:NO];
+    [refreshIntervalText setEnabled:NO];
+    [refreshIntervalText setDoubleValue:0];
+    [refreshInterval setDoubleValue:0];
+  }
+}
+
+- (void)selectStore:(id)sender
 {
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   [storeColor setColor:[store eventColor]];
   [storeTextColor setColor:[store textColor]];
   [storeDisplay setState:[store displayed]];
   [storeWritable setState:[store writable]];
+  [storeEnabled setState:[store enabled]];
   if ([[defaultStorePopUp titleOfSelectedItem] isEqual:[store description]])
     [removeButton setEnabled:NO];
   else
     [removeButton setEnabled:YES];
+  [self periodicSetupForStore:store];
 }
 
--(void)changeColor:(id)sender
+- (void)changeColor:(id)sender
 {
   NSColor *rgb = [[storeColor color] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   [store setEventColor:rgb];
 }
 
--(void)changeTextColor:(id)sender
+- (void)changeTextColor:(id)sender
 {
   NSColor *rgb = [[storeTextColor color] colorUsingColorSpaceName:NSCalibratedRGBColorSpace];
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   [store setTextColor:rgb];
 }
 
--(void)changeStart:(id)sender
+- (void)changeStart:(id)sender
 {
   int value = [dayStart intValue];
   if (value != [[ConfigManager globalConfig] integerForKey:FIRST_HOUR]) {
@@ -125,7 +172,7 @@
   }
 }
 
--(void)changeEnd:(id)sender
+- (void)changeEnd:(id)sender
 {
   int value = [dayEnd intValue];
   if (value != [[ConfigManager globalConfig] integerForKey:LAST_HOUR]) {
@@ -134,7 +181,7 @@
   }
 }
 
--(void)changeStep:(id)sender
+- (void)changeStep:(id)sender
 {
   double value = [minStep doubleValue];
   if (value * 60 != [[ConfigManager globalConfig] integerForKey:MIN_STEP]) {
@@ -143,26 +190,63 @@
   }
 }
 
--(void)selectDefaultStore:(id)sender
+- (void)changeInterval:(id)sender
+{
+  double value = [refreshInterval doubleValue];
+  id <PeriodicRefresh> store = (id <PeriodicRefresh>)[_sm storeForName:[storePopUp titleOfSelectedItem]];
+  [store setRefreshInterval:value * 3600];
+  [refreshIntervalText setDoubleValue:value];
+  [refreshInterval setDoubleValue:value];
+}
+
+- (void)selectDefaultStore:(id)sender
 {
   [_sm setDefaultStore:[defaultStorePopUp titleOfSelectedItem]];
   [self selectStore:nil];
 }
 
--(void)toggleDisplay:(id)sender
+- (void)toggleDisplay:(id)sender
 {
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   [store setDisplayed:[storeDisplay state]];
 }
 
--(void)toggleWritable:(id)sender
+- (void)toggleWritable:(id)sender
 {
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   [store setWritable:[storeWritable state]];
 }
 
+- (void)toggleRefresh:(id)sender
+{
+  id <PeriodicRefresh> store = (id <PeriodicRefresh>)[_sm storeForName:[storePopUp titleOfSelectedItem]];
+  [store setPeriodicRefresh:[storeRefresh state]];
+  [self periodicSetupForStore:store];
+}
+
+- (void)toggleEnabled:(id)sender
+{
+  id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
+  [store setEnabled:[storeEnabled state]];
+}
+
+- (void)toggleTooltip:(id)sender
+{
+  [[ConfigManager globalConfig] setInteger:[showTooltip state] forKey:TOOLTIP];
+}
+
+- (void)toggleShowDate:(id)sender
+{
+  [[ConfigManager globalConfig] setInteger:[showDateAppIcon state] forKey:APPICON_DATE];
+}
+
+- (void)toggleShowTime:(id)sender
+{
+  [[ConfigManager globalConfig] setInteger:[showTimeAppIcon state] forKey:APPICON_TIME];
+}
+
 /* We only allow the removal of non-default stores */
--(void)removeStore:(id)sender
+- (void)removeStore:(id)sender
 {
   id <AgendaStore> store = [_sm storeForName:[storePopUp titleOfSelectedItem]];
   ConfigManager *config = [ConfigManager globalConfig];
@@ -173,10 +257,10 @@
   [config removeObjectForKey:[store description]];
   [_sm removeStoreNamed:[store description]];
   /* FIXME : This could be done by registering STORES key */
-  [self _setupStores];
+  [self setupStores];
 }
 
--(void)createStore:(id)sender
+- (void)createStore:(id)sender
 {
   ConfigManager *config = [ConfigManager globalConfig];
   NSMutableArray *storeArray = [NSMutableArray arrayWithArray:[config objectForKey:STORES]];
@@ -187,13 +271,13 @@
     [_sm addStoreNamed:[storeName stringValue]];
     [storeArray addObject:[storeName stringValue]];
     [config setObject:storeArray forKey:STORES];
-    [self _setupStores];
+    [self setupStores];
   }
   [storeName setStringValue:@""];
   [createButton setEnabled:NO];
 }
 
--(void)selectItem:(id)sender
+- (void)selectItem:(id)sender
 {
   switch ([sender indexOfSelectedItem]) {
   case 0:
@@ -205,11 +289,14 @@
   case 2:
     [slot setContentView:storeFactory];
     break;
+  case 3:
+    [slot setContentView:uiPreferences];
+    break;
   }
   [itemPopUp setNextKeyView:[slot contentView]];
 }
 
--(void)controlTextDidChange:(NSNotification *)notification
+- (void)controlTextDidChange:(NSNotification *)notification
 {
   if ([notification object] == storeName) {
     if ([_sm storeForName:[storeName stringValue]] || ![[storeName stringValue] length])
@@ -218,5 +305,4 @@
       [createButton setEnabled:YES];
   }
 }
-
 @end
