@@ -1,44 +1,52 @@
-/* emacs buffer mode hint -*- objc -*- */
-
 #import <Foundation/Foundation.h>
 #import "AgendaStore.h"
 #import "StoreManager.h"
 #import "ConfigManager.h"
 #import "defines.h"
-#import "LocalStore.h"
-#import "iCalStore.h"
+#import "Event.h"
 
-static NSMutableDictionary *backends = nil;
+NSString * const SADataChangedInStoreManager = @"SADataDidChangedInStoreManager";
+static NSString * const PERSONAL_AGENDA = @"Personal Agenda";
+
+static NSMutableDictionary *backends;
+static StoreManager *singleton;
 
 @implementation StoreManager
-
-#define PERSONAL_AGENDA @"Personal Agenda"
-
 + (void)initialize
 {
-  NSArray *classes = GSObjCAllSubclassesOfClass([MemoryStore class]);
-  NSEnumerator *enumerator = [classes objectEnumerator];
+  NSArray *classes;
+  NSEnumerator *enumerator;
   Class backendClass;
 
-  if (backends == nil) {
+  if ([StoreManager class] == self) {
+    classes = GSObjCAllSubclassesOfClass([MemoryStore class]);
+    enumerator = [classes objectEnumerator];
     backends = [[NSMutableDictionary alloc] initWithCapacity:[classes count]];
     while ((backendClass = [enumerator nextObject])) {
-      if ([backendClass conformsToProtocol:@protocol(AgendaStore)])
+      if ([backendClass conformsToProtocol:@protocol(MemoryStore)])
 	[backends setObject:backendClass forKey:[backendClass storeTypeName]];
       else
 	NSLog(@"Can't register %@ as a store backend", [backendClass description]);
     }
+    singleton = [[StoreManager alloc] init];
   }
 }
+
 + (NSArray *)backends
 {
   return [backends allValues];
 }
+
 + (Class)backendForName:(NSString *)name
 {
   return [backends valueForKey:name];
 }
 
++ (StoreManager *)globalManager
+{
+  return singleton;
+}
+
 - (NSDictionary *)defaults
 {
   NSDictionary *local = [NSDictionary
@@ -56,22 +64,51 @@ static NSMutableDictionary *backends = nil;
   NSString *defaultStore;
   NSEnumerator *enumerator;
   NSString *stname;
+  Class backendClass;
   ConfigManager *config = [ConfigManager globalConfig];
+  id store;
 
   self = [super init];
   if (self) {
     [config registerDefaults:[self defaults]];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					     selector:@selector(dataChanged:)
+					         name:SADataChangedInStore
+					       object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					     selector:@selector(dataChanged:)
+					         name:SAElementAddedToStore
+					       object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					     selector:@selector(dataChanged:)
+					         name:SAElementRemovedFromStore
+					       object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					     selector:@selector(dataChanged:)
+					         name:SAElementUpdatedInStore
+					       object:nil];
+    [[NSNotificationCenter defaultCenter] addObserver:self 
+					     selector:@selector(dataChanged:)
+					         name:SAEnabledStatusChangedForStore
+					       object:nil];
+    _stores = [[NSMutableDictionary alloc] initWithCapacity:1];
+    _cache = [[NSMutableDictionary alloc] initWithCapacity:256];
+    /* Create user defined stores */
     storeArray = [config objectForKey:STORES];
     defaultStore = [config objectForKey:ST_DEFAULT];
-    _stores = [[NSMutableDictionary alloc] initWithCapacity:1];
     enumerator = [storeArray objectEnumerator];
     while ((stname = [enumerator nextObject]))
       [self addStoreNamed:stname];
+    /* Create automatic stores */
+    enumerator = [backends objectEnumerator];
+    while ((backendClass = [enumerator nextObject])) {
+      if (![backendClass isUserInstanciable] && [backendClass storeName]) {
+	store = [backendClass storeNamed:[backendClass storeName]];
+	[_stores setObject:store forKey:[backendClass storeName]];
+	NSLog(@"Added %@ to StoreManager", [backendClass storeName]);
+      }
+    }
     [self setDefaultStore:defaultStore];
-    [[NSNotificationCenter defaultCenter] addObserver:self 
-					  selector:@selector(dataChanged:) 
-					  name:SADataChangedInStore 
-					  object:nil];
   }
   return self;
 }
@@ -79,14 +116,17 @@ static NSMutableDictionary *backends = nil;
 - (void)dealloc
 {
   [[NSNotificationCenter defaultCenter] removeObserver:self];
+  [self synchronise];
   RELEASE(_defaultStore);
   [_stores release];
+  [_cache release];
   [super dealloc];
 }
 
 - (void)dataChanged:(NSNotification *)not
 {
-  [[NSNotificationCenter defaultCenter] postNotificationName:SADataChanged object:self];
+  [_cache removeAllObjects];
+  [[NSNotificationCenter defaultCenter] postNotificationName:SADataChangedInStoreManager object:self];
 }
 
 - (void)addStoreNamed:(NSString *)name
@@ -102,7 +142,6 @@ static NSMutableDictionary *backends = nil;
     if (store) {
       [_stores setObject:store forKey:name];
       NSLog(@"Added %@ to StoreManager", name);
-      [self dataChanged:nil];
     } else
       NSLog(@"Unable to initialize store %@", name);
   }
@@ -145,7 +184,8 @@ static NSMutableDictionary *backends = nil;
   id <AgendaStore> store;
 
   while ((store = [enumerator nextObject]))
-    [store write];
+    if ([store conformsToProtocol:@protocol(StoreBackend)])
+      [store write];
 }
 
 - (void)refresh
@@ -175,9 +215,11 @@ static NSMutableDictionary *backends = nil;
   id <AgendaStore> store;
 
   while ((store = [enumerator nextObject]))
-    [all addObjectsFromArray:[store events]];
+    if ([store enabled] && [store displayed])
+      [all addObjectsFromArray:[store events]];
   return all;
 }
+
 - (NSArray *)allTasks;
 {
   NSMutableArray *all = [NSMutableArray arrayWithCapacity:128];
@@ -185,8 +227,28 @@ static NSMutableDictionary *backends = nil;
   id <AgendaStore> store;
 
   while ((store = [enumerator nextObject]))
-    [all addObjectsFromArray:[store tasks]];
+    if ([store enabled] && [store displayed])
+      [all addObjectsFromArray:[store tasks]];
   return all;
 }
-@end
 
+- (NSSet *)scheduledAppointmentsForDay:(Date *)date
+{
+  NSMutableSet *dayEvents;
+  NSEnumerator *enumerator;
+  Event *event;
+
+  NSAssert(date != nil, @"No date specified, am I supposed to guess ?");
+  dayEvents = [_cache objectForKey:date];
+  if (dayEvents)
+    return dayEvents;
+
+  dayEvents = [NSMutableSet setWithCapacity:8];
+  enumerator = [[self allEvents] objectEnumerator];
+  while ((event = [enumerator nextObject]))
+    if ([event isScheduledForDay:date])
+      [dayEvents addObject:event];
+  [_cache setObject:dayEvents forKey:date];
+  return dayEvents;
+}
+@end
