@@ -2,7 +2,6 @@
 
 #import "Date.h"
 #import "Element.h"
-#import "AgendaStore.h"
 
 enum taskState
 {
@@ -25,3 +24,8 @@ enum taskState
 - (Date *)completionDate;
 - (void)setCompletionDate:(Date *)cd;
 @end
+
+@interface Task(iCalendar)
+- (id)initWithICalComponent:(icalcomponent *)ic;
+- (BOOL)updateICalComponent:(icalcomponent *)ic;
+@end
