@@ -1,7 +1,8 @@
 /* emacs buffer mode hint -*- objc -*- */
 
-#import "Task.h"
-#import "StoreManager.h"
+#import <Foundation/Foundation.h>
+
+@class Task;
 
 @interface TaskEditor : NSObject
 {
@@ -11,10 +12,10 @@
   id store;
   id state;
   id ok;
+  Task *_task;
 }
 
-- (BOOL)editTask:(Task *)data withStoreManager:(StoreManager *)sm;
++ (TaskEditor *)editorForTask:(Task *)task;
 - (void)validate:(id)sender;
 - (void)cancel:(id)sender;
-
 @end
