@@ -3,6 +3,8 @@
 #define FIRST_HOUR @"firstHour"
 #define LAST_HOUR @"lastHour"
 #define MIN_STEP @"minimumStep"
+#define TOOLTIP @"showTooltip"
+#define ALARMS @"activateAlarms"
 
 #define STORE_CLASSES @"storeClasses"
 
@@ -16,6 +18,9 @@
 #define ST_CALENDAR_URL @"storeCalendarURL"
 #define ST_TASK_URL @"storeTaskURL"
 #define ST_RW @"storeWritable"
-#define ST_REFRESH @"storeRefresh"
+#define ST_REFRESH @"storeAutomaticRefresh"
+#define ST_REFRESH_INTERVAL @"storeRefreshInterval"
 #define ST_DISPLAY @"storeDisplay"
-#define ST_VERSION @"storeVersion"
+#define ST_ENABLED @"storeEnabled"
+#define APPICON_DATE @"appIconShowDate"
+#define APPICON_TIME @"appIconShowTime"
