@@ -1,10 +1,20 @@
-#import <Foundation/Foundation.h>
+#import <AppKit/AppKit.h>
 #import "Event.h"
 #import "Task.h"
-#import "iCalStore.h"
+#import "AgendaStore.h"
 #import "WebDAVResource.h"
+#import "iCalTree.h"
 #import "defines.h"
 
+@interface iCalStore : MemoryStore <SharedStore, ConfigListener>
+{
+  iCalTree *_tree;
+  NSURL *_url;
+  NSTimer *_refreshTimer;
+  WebDAVResource *_resource;
+}
+@end
+
 @interface iCalStoreDialog : NSObject
 {
   IBOutlet id panel;
@@ -42,12 +52,19 @@
 }
 - (void)okClicked:(id)sender
 {
+  BOOL readable;
   NSURL *tmp;
   WebDAVResource *resource;
 
   tmp = [NSURL URLWithString:[url stringValue]];
   resource = AUTORELEASE([[WebDAVResource alloc] initWithURL:tmp]);
-  if ([resource readable])
+  readable = [resource readable];
+  /* Read will fail if there's no resource yet, try to create an empty one */
+  if (!readable) {
+    [resource writableWithData:[NSData data]];
+    readable = [resource readable];
+  }
+  if (readable)
     [NSApp stopModalWithCode:1];
   else {
     [error setStringValue:[NSString stringWithFormat:@"Unable to read from this URL : %@", [tmp propertyForKey:NSHTTPPropertyStatusReasonKey]]];
@@ -77,20 +94,21 @@
 @end
 
 @interface iCalStore(Private)
-- (void)fetchData:(id)object;
+- (void)fetchData;
 - (void)parseData:(NSData *)data;
-- (void)initTimer:(id)object;
+- (void)initTimer;
 - (void)initStoreAsync:(id)object;
 @end
 
 @implementation iCalStore
 - (NSDictionary *)defaults
 {
-  return [NSDictionary dictionaryWithObjectsAndKeys:
-			 [NSArchiver archivedDataWithRootObject:[NSColor blueColor]], ST_COLOR,
-			 [NSArchiver archivedDataWithRootObject:[NSColor darkGrayColor]], ST_TEXT_COLOR,
+  return [NSDictionary dictionaryWithObjectsAndKeys:[[NSColor blueColor] description], ST_COLOR,
+		       [[NSColor darkGrayColor] description], ST_TEXT_COLOR,
 		       [NSNumber numberWithBool:NO], ST_RW,
 		       [NSNumber numberWithBool:YES], ST_DISPLAY,
+		       [NSNumber numberWithBool:NO], ST_REFRESH,
+		       [NSNumber numberWithBool:YES], ST_ENABLED,
 		       nil, nil];
 }
 
@@ -99,11 +117,21 @@
   self = [super initWithName:name];
   if (self) {
     _tree = [iCalTree new];
+    _url = [[NSURL alloc] initWithString:[_config objectForKey:ST_URL]];
+    _resource = [[WebDAVResource alloc] initWithURL:_url];
+    [_config registerClient:self forKey:ST_REFRESH];
+    [_config registerClient:self forKey:ST_REFRESH_INTERVAL];
     [NSThread detachNewThreadSelector:@selector(initStoreAsync:) toTarget:self withObject:nil];
+    [self initTimer];
   }
   return self;
 }
 
++ (BOOL)isUserInstanciable
+{
+  return YES;
+}
+
 + (BOOL)registerWithName:(NSString *)name
 {
   ConfigManager *cm;
@@ -111,24 +139,21 @@
   NSURL *storeURL;
   BOOL writable = NO;
   WebDAVResource *resource;
-  NSData *data;
 
   dialog = [[iCalStoreDialog alloc] initWithName:name];
   if ([dialog show] == YES) {
     storeURL = [NSURL URLWithString:[dialog url]];
     resource = [[WebDAVResource alloc] initWithURL:storeURL];
-    data = [resource get];
     writable = NO;
-    if (data) {
-      writable = [resource writableWithData:data];
-      [data release];
-    }
+    if ([resource get])
+      writable = [resource writableWithData:[resource data]];
     [resource release];
     [dialog release];
     cm = [[ConfigManager alloc] initForKey:name withParent:nil];
     [cm setObject:[storeURL description] forKey:ST_URL];
     [cm setObject:[[self class] description] forKey:ST_CLASS];
     [cm setObject:[NSNumber numberWithBool:writable] forKey:ST_RW];
+    [cm release];
     return YES;
   }
   [dialog release];
@@ -142,7 +167,9 @@
 
 - (void)dealloc
 {
+  [_config unregisterClient:self];
   [_refreshTimer invalidate];
+  [_refreshTimer release];
   [self write];
   DESTROY(_resource);
   DESTROY(_url);
@@ -159,8 +186,7 @@
 {
   if ([_tree add:elt]) {
     [super add:elt];
-    if (![_url isFileURL])
-      [self write];
+    [self write];
   }
 }
 
@@ -168,8 +194,7 @@
 {
   if ([_tree remove:elt]) {
     [super remove:elt];
-    if (![_url isFileURL])
-      [self write];
+    [self write];
   }
 }
 
@@ -177,29 +202,24 @@
 {
   if ([_tree update:(Event *)elt]) {
     [super update:elt];
-    if (![_url isFileURL])
-      [self write];
+    [self write];
   }
 }
 
-- (BOOL)read
+- (void)read
 {
-  [self fetchData:nil];
-  return [_resource dataChanged];
+  [self fetchData];
 }
 
 - (BOOL)write
 {
   NSData *data;
-  NSData *read;
 
   if (![self modified] || ![self writable])
     return YES;
   data = [_tree iCalTreeAsData];
   if (data) {
-    read = [_resource put:data];
-    DESTROY(read);
-    if ([_resource status] == NSURLHandleLoadSucceeded) {
+    if ([_resource put:data attributes:nil]) {
       [_resource updateAttributes];
       [self setModified:NO];
       NSLog(@"iCalStore written to %@", [_url absoluteString]);
@@ -215,13 +235,44 @@
   }
   return YES;
 }
+
+- (BOOL)periodicRefresh
+{
+  if ([_config objectForKey:ST_REFRESH])
+    return [[_config objectForKey:ST_REFRESH] boolValue];
+  return NO;
+}
+- (void)setPeriodicRefresh:(BOOL)periodic
+{
+  [_config setObject:[NSNumber numberWithBool:periodic] forKey:ST_REFRESH];
+}
+- (NSTimeInterval)refreshInterval
+{
+  if ([_config objectForKey:ST_REFRESH_INTERVAL])
+    return [_config integerForKey:ST_REFRESH_INTERVAL];
+  return 60 * 30;
+}
+- (void)setRefreshInterval:(NSTimeInterval)interval
+{
+  [_config setInteger:interval forKey:ST_REFRESH_INTERVAL];
+}
+- (void)config:(ConfigManager*)config dataDidChangedForKey:(NSString *)key
+{
+  [self initTimer];
+}
 @end
 
 
 @implementation iCalStore(Private)
-- (void)fetchData:(id)object
+- (void)fetchData
 {
-  [self parseData:AUTORELEASE([_resource get])];
+  if ([_resource get])
+    if ([NSThread isMainThread])
+      [self parseData:[_resource data]];
+    else
+      [self performSelectorOnMainThread:@selector(parseData:) withObject:[_resource data] waitUntilDone:NO];
+  else
+    [self setEnabled:NO];
 }
 - (void)parseData:(NSData *)data
 {
@@ -232,25 +283,29 @@
   } else
     NSLog(@"Couldn't parse data from %@", [_url absoluteString]);
 }
-- (void)initTimer:(id)object
+- (void)initTimer
 {
-  if ([_config objectForKey:ST_REFRESH])
-    _minutesBeforeRefresh = [_config integerForKey:ST_REFRESH];
-  else
-    _minutesBeforeRefresh = 30;
-  _refreshTimer = [[NSTimer alloc] initWithFireDate:nil
-				   interval:_minutesBeforeRefresh * 60
-				   target:self selector:@selector(refreshData:) 
-				   userInfo:nil repeats:YES];
-  [[NSRunLoop currentRunLoop] addTimer:_refreshTimer forMode:NSDefaultRunLoopMode];
+  if (nil != _refreshTimer) {
+    [_refreshTimer invalidate];
+    DESTROY(_refreshTimer);
+  }
+  if ([self periodicRefresh]) {
+    _refreshTimer = [[NSTimer alloc] initWithFireDate:nil
+				             interval:[self refreshInterval]
+                   			       target:self
+				             selector:@selector(refreshData:) 
+				             userInfo:nil 
+				              repeats:YES];
+    [[NSRunLoop currentRunLoop] addTimer:_refreshTimer forMode:NSDefaultRunLoopMode];
+    NSLog(@"Store %@ will refresh every %d seconds", [self description], (int)[self refreshInterval]);
+  } else {
+    NSLog(@"Store %@ automatic refresh disabled", [self description]);
+  }
 }
 - (void)initStoreAsync:(id)object
 {
   NSAutoreleasePool *pool = [NSAutoreleasePool new];
-  _url = [[NSURL alloc] initWithString:[_config objectForKey:ST_URL]];
-  _resource = [[WebDAVResource alloc] initWithURL:_url];
-  [self performSelectorOnMainThread:@selector(fetchData:) withObject:nil waitUntilDone:YES];
-  [self performSelectorOnMainThread:@selector(initTimer:) withObject:nil waitUntilDone:YES];
+  [self fetchData];
   [pool release];
 }
 @end
